@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_arm.c,v 1.68 2007/03/26 02:01:35 debug Exp $
+ *  $Id: cpu_arm.c,v 1.71 2007/06/15 00:41:21 debug Exp $
  *
  *  ARM CPU emulation.
  *
@@ -39,6 +39,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include "arm_cpu_types.h"
 #include "cpu.h"
@@ -49,11 +50,15 @@
 #include "of.h"
 #include "settings.h"
 #include "symbol.h"
+#include "timer.h"
+#include "useremul.h"
 
 #define DYNTRANS_32
 #include "tmp_arm_head.c"
 
 
+extern int native_code_translation_enabled;
+
 /*  ARM symbolic register names and condition strings:  */
 static char *arm_regname[N_ARM_REGS] = ARM_REG_NAMES;
 static char *arm_condition_string[16] = ARM_CONDITION_STRINGS;
@@ -190,6 +195,9 @@ int arm_cpu_new(struct cpu *cpu, struct memory *mem,
 		/*  FIQ: TODO  */
         }
 
+	if (native_code_translation_enabled)
+		cpu->sampling_timer = timer_add(CPU_SAMPLE_TIMER_HZ,
+		    arm_timer_sample_tick, cpu);
 
 	return 1;
 }
