@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ppc.c,v 1.68 2007/03/26 02:01:36 debug Exp $
+ *  $Id: cpu_ppc.c,v 1.70 2007/06/15 00:41:21 debug Exp $
  *
  *  PowerPC/POWER CPU emulation.
  */
@@ -49,11 +49,16 @@
 #include "ppc_spr_strings.h"
 #include "settings.h"
 #include "symbol.h"
+#include "timer.h"
+#include "useremul.h"
+
 
 #define	DYNTRANS_DUALMODE_32
 #include "tmp_ppc_head.c"
 
 
+extern int native_code_translation_enabled;
+
 void ppc_pc_to_pointers(struct cpu *);
 void ppc32_pc_to_pointers(struct cpu *);
 
@@ -233,6 +238,10 @@ int ppc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 		interrupt_handler_register(&template);
 	}
 
+	if (native_code_translation_enabled)
+		cpu->sampling_timer = timer_add(CPU_SAMPLE_TIMER_HZ,
+		    ppc_timer_sample_tick, cpu);
+
 	return 1;
 }
 
