@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sparc.c,v 1.44 2007/03/26 02:18:44 debug Exp $
+ *  $Id: cpu_sparc.c,v 1.46 2007/06/07 15:36:24 debug Exp $
  *
  *  SPARC CPU emulation.
  */
@@ -41,6 +41,7 @@
 #include "misc.h"
 #include "settings.h"
 #include "symbol.h"
+#include "timer.h"
 
 
 #define	DYNTRANS_DUALMODE_32
@@ -48,6 +49,8 @@
 #include "tmp_sparc_head.c"
 
 
+extern int native_code_translation_enabled;
+
 static char *sparc_regnames[N_SPARC_REG] = SPARC_REG_NAMES;
 static char *sparc_pregnames[N_SPARC_PREG] = SPARC_PREG_NAMES;
 static char *sparc_regbranch_names[N_SPARC_REGBRANCH_TYPES] =
@@ -168,6 +171,10 @@ int sparc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 		    cpu->cd.sparc.r[i]);
 	/*  TODO: Handler for writes to the zero register!  */
 
+	if (native_code_translation_enabled)
+		cpu->sampling_timer = timer_add(CPU_SAMPLE_TIMER_HZ,
+		    sparc_timer_sample_tick, cpu);
+
 	return 1;
 }
 
@@ -267,7 +274,8 @@ void sparc_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 					continue;
 				}
 				debug("%s=", sparc_regnames[i]);
-				debug("0x%08x", (int) cpu->cd.sparc.r[i]);
+				debug("0x%08"PRIx32,
+				    (uint32_t) cpu->cd.sparc.r[i]);
 				if ((i & 3) < 3)
 					debug("  ");
 				else
@@ -459,7 +467,7 @@ int sparc_cpu_disassemble_instr(struct cpu *cpu, unsigned char *instr,
 	iword = *(uint32_t *)&instr[0];
 	iword = BE32_TO_HOST(iword);
 
-	debug(": %08x", iword);
+	debug(": %08"PRIx32, iword);
 
 	if (running && cpu->delay_slot)
 		debug(" (d)");
