@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory_sh.c,v 1.19 2007/04/16 15:11:31 debug Exp $
+ *  $Id: memory_sh.c,v 1.23 2007/06/05 06:41:30 debug Exp $
  */
 
 #include <stdio.h>
@@ -153,7 +153,11 @@ static int translate_via_mmu(struct cpu *cpu, uint32_t vaddr,
 	if (flags & FLAG_INSTR) {
 		/*
 		 *  Instruction access:
-		 *
+		 */
+#if 0
+		/*  NOTE: Emulating the ITLB as exact as this is not
+			necessary... so I'm disabling it for now.  */
+		/*
 		 *  If a matching entry wasn't found in the ITLB, but in the
 		 *  UTLB, then copy it to a random place in the ITLB.
 		 */
@@ -171,6 +175,7 @@ static int translate_via_mmu(struct cpu *cpu, uint32_t vaddr,
 			cpu->cd.sh.itlb_hi[r] = cpu->cd.sh.utlb_hi[i];
 			cpu->cd.sh.itlb_lo[r] = cpu->cd.sh.utlb_lo[i];
 		}
+#endif
 
 		/*  Permission checks:  */
 		if (cpu->cd.sh.sr & SH_SR_MD)
@@ -305,8 +310,12 @@ int sh_translate_v2p(struct cpu *cpu, uint64_t vaddr64, uint64_t *return_paddr,
 	}
 
 	/*  TODO  */
-	fatal("Unimplemented SH vaddr 0x%08"PRIx32"\n", (uint32_t)vaddr);
-	exit(1);
+
+	/*  The ugly 'if' is just here to fool Compaq CC.  */
+	if (!(flags & FLAG_NOEXCEPTIONS)) {
+		fatal("Unimplemented SH vaddr 0x%08"PRIx32"\n", vaddr);
+		exit(1);
+	}
 
 	return 0;
 }
