@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: dev_adb.c,v 1.12 2007/01/28 11:29:52 debug Exp $
+ *  $Id: dev_adb.c,v 1.14 2007/06/15 18:44:18 debug Exp $
  *
- *  ADB (Apple Desktop Bus) controller.
+ *  COMMENT: Apple Desktop Bus (ADB) controller
  *
  *  Based on intuition from reverse-engineering NetBSD/macppc source code,
  *  so it probably only works with that OS.
@@ -438,12 +438,9 @@ DEVICE_ACCESS(adb)
 
 DEVINIT(adb)
 {
-	struct adb_data *d = malloc(sizeof(struct adb_data));
+	struct adb_data *d;
 
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(d = malloc(sizeof(struct adb_data)));
 	memset(d, 0, sizeof(struct adb_data));
 
 	INTERRUPT_CONNECT(devinit->interrupt_path, d->irq);
@@ -453,7 +450,7 @@ DEVINIT(adb)
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_ADB_LENGTH, dev_adb_access, d, DM_DEFAULT, NULL);
 	machine_add_tickfunction(devinit->machine, dev_adb_tick, d,
-	    TICK_SHIFT, 0.0);
+	    TICK_SHIFT);
 
 	return 1;
 }
