@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_bt459.c,v 1.67 2006/12/30 13:30:57 debug Exp $
+ *  $Id: dev_bt459.c,v 1.70 2007/06/15 18:44:19 debug Exp $
  *  
- *  Brooktree 459 vdac, used by TURBOchannel graphics cards.
+ *  COMMENT: Brooktree BT459, used by TURBOchannel graphics cards
  */
 
 #include <stdio.h>
@@ -145,7 +145,7 @@ static void bt459_update_X_cursor(struct cpu *cpu, struct bt459_data *d)
 	 */
 
 #ifdef WITH_X11
-	if (cpu->machine->use_x11 && d->vfb_data->fb_window != NULL) {
+	if (cpu->machine->x11_md.in_use && d->vfb_data->fb_window != NULL) {
 		for (y=0; y<=ymax; y++) {
 			for (x=0; x<=xmax; x+=4) {
 				struct fb_window *win = d->vfb_data->fb_window;
@@ -284,7 +284,7 @@ DEVICE_TICK(bt459)
 
 DEVICE_ACCESS(bt459_irq)
 {
-	struct bt459_data *d = (struct bt459_data *) extra;
+	struct bt459_data *d = extra;
 	uint64_t idata = 0, odata = 0;
 
 	if (writeflag == MEM_WRITE)
@@ -305,12 +305,9 @@ DEVICE_ACCESS(bt459_irq)
 }
 
 
-/*
- *  dev_bt459_access():
- */
 DEVICE_ACCESS(bt459)
 {
-	struct bt459_data *d = (struct bt459_data *) extra;
+	struct bt459_data *d = extra;
 	uint64_t idata = 0, odata = 0;
 	int btaddr, old_cursor_on = d->cursor_on, modified;
 
@@ -528,12 +525,9 @@ void dev_bt459_init(struct machine *machine, struct memory *mem,
 	uint64_t baseaddr, uint64_t baseaddr_irq, struct vfb_data *vfb_data,
 	int planes, char *irq_path, int type)
 {
-	struct bt459_data *d = malloc(sizeof(struct bt459_data));
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	struct bt459_data *d;
 
+	CHECK_ALLOCATION(d = malloc(sizeof(struct bt459_data)));
 	memset(d, 0, sizeof(struct bt459_data));
 
 	INTERRUPT_CONNECT(irq_path, d->irq);
@@ -584,6 +578,6 @@ void dev_bt459_init(struct machine *machine, struct memory *mem,
 		    dev_bt459_irq_access, (void *)d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(machine, dev_bt459_tick, d,
-	    BT459_TICK_SHIFT, 0.0);
+	    BT459_TICK_SHIFT);
 }
 
