@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: dev_i80321.c,v 1.21 2007/04/28 09:19:52 debug Exp $
+ *  $Id: dev_i80321.c,v 1.23 2007/06/15 19:11:15 debug Exp $
  *
- *  Intel i80321 (ARM) core functionality.
+ *  COMMENT: Intel i80321 (ARM) core functionality
  *
  *	o)  Interrupt controller
  *	o)  Timer
@@ -284,18 +284,15 @@ ret:
 
 DEVINIT(i80321)
 {
-	struct i80321_data *d = malloc(sizeof(struct i80321_data));
+	struct i80321_data *d;
 	uint32_t memsize = devinit->machine->physical_ram_in_mb * 1048576;
 	uint32_t base;
 	char tmpstr[300];
-	int i;
 	struct cpu *cpu = devinit->machine->cpus[devinit->
 	    machine->bootstrap_cpu];
+	int i;
 
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(d = malloc(sizeof(struct i80321_data)));
 	memset(d, 0, sizeof(struct i80321_data));
 
 	/*  Connect to the CPU interrupt pin:  */
@@ -354,7 +351,7 @@ DEVINIT(i80321)
 	d->timer = timer_add(d->hz, tmr0_tick, d);
 
 	machine_add_tickfunction(devinit->machine, dev_i80321_tick,
-	    d, TICK_SHIFT, 0.0);
+	    d, TICK_SHIFT);
 
 	devinit->return_ptr = d->pci_bus;
 
