@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_kn02.c,v 1.26 2007/02/10 14:20:52 debug Exp $
+ *  $Id: dev_kn02.c,v 1.28 2007/06/15 19:11:15 debug Exp $
  *  
- *  KN02 mainbus (TurboChannel interrupt controller).
+ *  COMMENT: DEC KN02 mainbus (TurboChannel interrupt controller)
  *
  *  Used in DECstation type 2 ("3MAX").  See include/dec_kn02.h for more info.
  */
@@ -44,6 +44,8 @@
 #include "misc.h"
 
 
+#include "dec_kn02.h"
+
 #define	DEV_KN02_LENGTH		0x1000
 
 
@@ -96,6 +98,7 @@ DEVICE_ACCESS(kn02)
 		if (writeflag==MEM_READ) {
 			odata = d->csr[0] + (d->csr[1] << 8) +
 			    (d->csr[2] << 16) + (d->csr[3] << 24);
+
 			/* debug("[ kn02: read from CSR: 0x%08x ]\n", odata); */
 		} else {
 			/*
@@ -146,13 +149,10 @@ DEVICE_ACCESS(kn02)
 DEVINIT(kn02)
 {
 	struct kn02_data *d;
+	uint32_t csr;
 	int i;
 
-	d = malloc(sizeof(struct kn02_data));
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(d = malloc(sizeof(struct kn02_data)));
 	memset(d, 0, sizeof(struct kn02_data));
 
 	/*  Connect the KN02 to a specific MIPS CPU interrupt line:  */
@@ -173,6 +173,16 @@ DEVINIT(kn02)
 		interrupt_handler_register(&template);
 	}
 
+	/*
+	 *  Set initial value of the CSR. Note: If the KN02_CSR_NRMMOD bit
+	 *  is not set, the 5000/200 PROM image loops forever.
+	 */
+	csr = KN02_CSR_NRMMOD;
+	d->csr[0] = csr;
+	d->csr[1] = csr >> 8;
+	d->csr[2] = csr >> 16;
+	d->csr[3] = csr >> 24;
+
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_KN02_LENGTH, dev_kn02_access, d,
 	    DM_DYNTRANS_OK, &d->csr[0]);
