@@ -25,9 +25,10 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_mc146818.c,v 1.95 2007/02/03 16:55:55 debug Exp $
+ *  $Id: dev_mc146818.c,v 1.99 2007/06/15 19:57:33 debug Exp $
  *  
- *  MC146818 real-time clock, used by many different machines types.
+ *  COMMENT: MC146818 real-time clock
+ *
  *  (DS1687 as used in some other machines is also similar to the MC146818.)
  *
  *  This device contains Date/time, the machine's ethernet address (on
@@ -58,7 +59,7 @@
 
 /*  #define MC146818_DEBUG  */
 
-#define	TICK_SHIFT	14
+#define	MC146818_TICK_SHIFT	14
 
 
 /*  256 on DECstation, SGI uses reg at 72*4 as the Century  */
@@ -119,13 +120,6 @@ DEVICE_TICK(mc146818)
 	int pti = d->pending_timer_interrupts;
 
 	if ((d->reg[MC_REGB * 4] & MC_REGB_PIE) && pti > 0) {
-		static int warned = 0;
-		if (pti > 1500 && !warned) {
-			warned = 1;
-			fatal("[ WARNING: MC146818 interrupts lost, "
-			    "host too slow? ]\n");
-		}
-
 #if 0
 		/*  For debugging, to see how much the interrupts are
 		    lagging behind the real clock:  */
@@ -157,9 +151,7 @@ DEVICE_TICK(mc146818)
  *  It seems like JAZZ machines accesses the mc146818 by writing one byte to
  *  0x90000070 and then reading or writing another byte at 0x......0004000.
  */
-int dev_mc146818_jazz_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(mc146818_jazz)
 {
 	struct mc_data *d = extra;
 
@@ -284,21 +276,15 @@ static void mc146818_update_time(struct mc_data *d)
 }
 
 
-/*
- *  dev_mc146818_access():
- *
- *  TODO: This access function only handles 8-bit accesses!
- */
-int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
-	uint64_t r, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(mc146818)
 {
+	struct mc_data *d = extra;
 	struct tm *tmp;
 	time_t timet;
-	struct mc_data *d = extra;
-	int relative_addr = r;
 	size_t i;
 
+	/*  NOTE/TODO: This access function only handles 8-bit accesses!  */
+
 	relative_addr /= d->addrdiv;
 
 	/*  Different ways of accessing the registers:  */
@@ -607,12 +593,7 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 	int i, dev_len;
 	struct mc_data *d;
 
-	d = malloc(sizeof(struct mc_data));
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
-
+	CHECK_ALLOCATION(d = malloc(sizeof(struct mc_data)));
 	memset(d, 0, sizeof(struct mc_data));
 
 	d->access_style  = access_style;
@@ -635,9 +616,9 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 	}
 
 	if (access_style == MC146818_DEC) {
-	/*  Station Ethernet Address, on DECstation 3100:  */
-	for (i=0; i<6; i++)
-		ether_address[i] = 0x10 * (i+1);
+		/*  Station Ethernet Address, on DECstation 3100:  */
+		for (i=0; i<6; i++)
+			ether_address[i] = 0x10 * (i+1);
 
 		d->reg[0x01] = ether_address[0];
 		d->reg[0x05] = ether_address[1];
@@ -704,6 +685,6 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 	mc146818_update_time(d);
 
 	machine_add_tickfunction(machine, dev_mc146818_tick, d,
-	    TICK_SHIFT, 0.0);
+	    MC146818_TICK_SHIFT);
 }
 
