@@ -25,17 +25,18 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: dev_mk48txx.c,v 1.4 2007/02/03 20:14:23 debug Exp $
+ *  $Id: dev_mk48txx.c,v 1.8 2007/06/15 19:57:33 debug Exp $
  *
- *  Mostek MK48Txx Real Time Clock.
+ *  COMMENT: Mostek MK48Txx Real Time Clock
  *
- *  TODO: Actually implement it. This is just enough to fool NetBSD/mvmeppc
- *        into thinking that the clock exists.
+ *  TODO:
+ *	Only the MK48T08 is implemented so far.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #include "cpu.h"
 #include "device.h"
@@ -48,50 +49,79 @@
 #include "mk48txxreg.h"
 
 
-#define	MK48TXX_LEN		16
+#define	MK48TXX_LEN		MK48T08_CLKSZ
+
+#define	BCD(x)	((((x) / 10) << 4) + ((x) % 10))
 
 struct mk48txx_data {
-	unsigned char	reg[MK48TXX_LEN];
+	uint8_t		reg[MK48TXX_LEN];
 };
 
 
-/*
- *  dev_mk48txx_access():
- */
+void mk48txx_update_regs(struct mk48txx_data *d)
+{
+	struct tm *tmp;
+	time_t timet;
+
+	timet = time(NULL);
+	tmp = gmtime(&timet);
+
+	d->reg[MK48T08_CLKOFF + MK48TXX_ISEC] = BCD(tmp->tm_sec);
+	d->reg[MK48T08_CLKOFF + MK48TXX_IMIN] = BCD(tmp->tm_min);
+	d->reg[MK48T08_CLKOFF + MK48TXX_IHOUR] = BCD(tmp->tm_hour);
+	d->reg[MK48T08_CLKOFF + MK48TXX_IWDAY] = tmp->tm_wday + 1;
+	d->reg[MK48T08_CLKOFF + MK48TXX_IDAY] = BCD(tmp->tm_mday);
+	d->reg[MK48T08_CLKOFF + MK48TXX_IMON] = BCD(tmp->tm_mon + 1);
+	d->reg[MK48T08_CLKOFF + MK48TXX_IYEAR] = BCD(tmp->tm_year % 100);
+}
+
+
 DEVICE_ACCESS(mk48txx)
 {
-	struct mk48txx_data *d = (struct mk48txx_data *) extra;
+	struct mk48txx_data *d = extra;
 	uint64_t idata = 0, odata = 0;
 
 	if (writeflag == MEM_WRITE)
 		idata = memory_readmax64(cpu, data, len);
 
+	if (writeflag == MEM_READ)
+		odata = d->reg[relative_addr];
+
+	if (relative_addr < MK48T08_CLKOFF ||
+	    relative_addr >= MK48T08_CLKOFF + MK48TXX_ISEC) {
+		/*  Reads and writes to the RAM part of the mk48txx, or
+		    the clock registers, are OK:  */
+		if (writeflag == MEM_WRITE)
+			d->reg[relative_addr] = idata;
+		goto ret;
+	}
+
 	switch (relative_addr) {
 
-	case MK48TXX_ICSR:
-		if (writeflag == MEM_READ)
-			odata = d->reg[relative_addr];
-		else
+	case MK48T08_CLKOFF + MK48TXX_ICSR:
+		if (writeflag == MEM_WRITE) {
+			if ((idata & MK48TXX_CSR_READ) &&
+			    !(d->reg[relative_addr] & MK48TXX_CSR_READ)) {
+				/*  Switching the read bit from 0 to 1 causes
+				    registers to be "froozen". In the emulator,
+				    simply updating them with data from the
+				    host should be good enough.  */
+				mk48txx_update_regs(d);
+			}
 			d->reg[relative_addr] = idata;
-		break;
-
-	case MK48TXX_ISEC:
-{
-/*  TODO: this is just a hack for NetBSD/mvmeppc mkclock detection :(  */
-static int x = 0;
-odata = x >> 1;
-x++;
-}
+		}
 		break;
 
 	default:if (writeflag == MEM_READ)
-			fatal("[ mk48txx: unimplemented READ from offset %i ]"
+			fatal("[ mk48txx: unimplemented READ from offset 0x%x ]"
 			    "\n", (int)relative_addr);
 		else
-			fatal("[ mk48txx: unimplemented WRITE to offset %i: "
+			fatal("[ mk48txx: unimplemented WRITE to offset 0x%x: "
 			    "0x%x ]\n", (int)relative_addr, (int)idata);
+		exit(1);
 	}
 
+ret:
 	if (writeflag == MEM_READ)
 		memory_writemax64(cpu, data, len, odata);
 
@@ -101,13 +131,13 @@ x++;
 
 DEVINIT(mk48txx)
 {
-	struct mk48txx_data *d = malloc(sizeof(struct mk48txx_data));
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	struct mk48txx_data *d;
+
+	CHECK_ALLOCATION(d = malloc(sizeof(struct mk48txx_data)));
 	memset(d, 0, sizeof(struct mk48txx_data));
 
+	mk48txx_update_regs(d);
+
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, MK48TXX_LEN, dev_mk48txx_access, (void *)d,
 	    DM_DEFAULT, NULL);
