@@ -25,9 +25,11 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pcic.c,v 1.18 2007/01/21 21:02:57 debug Exp $
+ *  $Id: dev_pcic.c,v 1.20 2007/06/15 19:57:33 debug Exp $
  *
- *  Intel 82365SL PC Card Interface Controller (called "pcic" by NetBSD).
+ *  COMMENT: Intel 82365SL PC Card Interface Controller
+ *
+ *  (Called "pcic" by NetBSD.)
  *
  *  TODO: Lots of stuff. This is just a quick hack. Don't rely on it.
  */
@@ -39,6 +41,7 @@
 #include "cpu.h"
 #include "device.h"
 #include "emul.h"
+#include "interrupt.h"
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
@@ -52,14 +55,11 @@
 #define	DEV_PCIC_LENGTH		2
 
 struct pcic_data {
-	int		irq_nr;
-	int		regnr;
+	struct interrupt	irq;
+	int			regnr;
 };
 
 
-/*
- *  dev_pcic_cis_access():
- */
 DEVICE_ACCESS(pcic_cis)
 {
 	/*  struct pcic_data *d = (struct pcic_data *) extra;  */
@@ -140,12 +140,9 @@ DEVICE_ACCESS(pcic_cis)
 }
 
 
-/*
- *  dev_pcic_access():
- */
 DEVICE_ACCESS(pcic)
 {
-	struct pcic_data *d = (struct pcic_data *) extra;
+	struct pcic_data *d = extra;
 	uint64_t idata = 0, odata = 0;
 	int controller_nr, socket_nr;
 
@@ -156,14 +153,17 @@ DEVICE_ACCESS(pcic)
 	socket_nr = d->regnr & 0x40? 1 : 0;
 
 	switch (relative_addr) {
+
 	case 0:	/*  Register select:  */
 		if (writeflag == MEM_WRITE)
 			d->regnr = idata;
 		else
 			odata = d->regnr;
 		break;
+
 	case 1:	/*  Register access:  */
 		switch (d->regnr & 0x3f) {
+
 		case PCIC_IDENT:
 			/*  This causes sockets A and B to be present on
 			    controller 0, and only socket A on controller 1.  */
@@ -178,15 +178,18 @@ DEVICE_ACCESS(pcic)
 			odata = PCIC_INTR_IRQ3;
 			break;
 #endif
+
 		case PCIC_CSC:
 			odata = PCIC_CSC_GPI;
 			break;
+
 		case PCIC_IF_STATUS:
 			odata = PCIC_IF_STATUS_READY
 			    | PCIC_IF_STATUS_POWERACTIVE;
 			if (controller_nr == 0 && socket_nr == 0)
 				odata |= PCIC_IF_STATUS_CARDDETECT_PRESENT;
 			break;
+
 		default:
 			if (writeflag == MEM_WRITE) {
 				debug("[ pcic: unimplemented write to "
@@ -213,14 +216,12 @@ DEVICE_ACCESS(pcic)
 DEVINIT(pcic)
 {
 	char tmpstr[200];
-	struct pcic_data *d = malloc(sizeof(struct pcic_data));
+	struct pcic_data *d;
 
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(d = malloc(sizeof(struct pcic_data)));
 	memset(d, 0, sizeof(struct pcic_data));
-	d->irq_nr = devinit->irq_nr;
+
+	INTERRUPT_CONNECT(devinit->interrupt_path, d->irq);
 
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_PCIC_LENGTH,
