@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_vr41xx.c,v 1.43 2007/01/21 21:02:57 debug Exp $
+ *  $Id: dev_vr41xx.c,v 1.46 2007/06/15 19:57:34 debug Exp $
  *  
- *  VR41xx (VR4122 and VR4131) misc functions.
+ *  COMMENT: VR41xx (VR4122 and VR4131) misc functions
  *
  *  This is just a big hack.
  *
@@ -394,7 +394,7 @@ static void vr41xx_keytick(struct cpu *cpu, struct vr41xx_data *d)
  */
 static void timer_tick(struct timer *timer, void *extra)
 {
-	struct vr41xx_data *d = (struct vr41xx_data *) extra;
+	struct vr41xx_data *d = extra;
 	d->pending_timer_interrupts ++;
 }
 
@@ -406,7 +406,7 @@ DEVICE_TICK(vr41xx)
 	if (d->pending_timer_interrupts > 0)
 		INTERRUPT_ASSERT(d->timer_irq);
 
-	if (cpu->machine->use_x11)
+	if (cpu->machine->x11_md.in_use)
 		vr41xx_keytick(cpu, d);
 }
 
@@ -481,7 +481,7 @@ static uint64_t vr41xx_kiu(struct cpu *cpu, int ofs, uint64_t idata,
 
 DEVICE_ACCESS(vr41xx)
 {
-	struct vr41xx_data *d = (struct vr41xx_data *) extra;
+	struct vr41xx_data *d = extra;
 	uint64_t idata = 0, odata = 0;
 	int regnr;
 	int revision = 0;
@@ -674,15 +674,12 @@ ret:
 struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 	struct memory *mem, int cpumodel)
 {
+	struct vr41xx_data *d;
 	uint64_t baseaddr = 0;
 	char tmps[300];
 	int i;
-	struct vr41xx_data *d = malloc(sizeof(struct vr41xx_data));
 
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(d = malloc(sizeof(struct vr41xx_data)));
 	memset(d, 0, sizeof(struct vr41xx_data));
 
 	/*  Connect to MIPS irq 2:  */
@@ -737,7 +734,7 @@ struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 	    machine->path, machine->bootstrap_cpu, VRIP_INTR_KIU);
 	INTERRUPT_CONNECT(tmps, d->kiu_irq);
 
-	if (machine->use_x11)
+	if (machine->x11_md.in_use)
 		machine->main_console_handle = d->kiu_console_handle;  
 
 	switch (cpumodel) {
@@ -797,7 +794,7 @@ struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 	device_add(machine, tmps);
 
 	machine_add_tickfunction(machine, dev_vr41xx_tick, d,
-	    DEV_VR41XX_TICKSHIFT, 0.0);
+	    DEV_VR41XX_TICKSHIFT);
 
 	/*  Some machines (?) use ISA space at 0x15000000 instead of
 	    0x14000000, eg IBM WorkPad Z50.  */
