@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_z8530.c,v 1.13 2007/01/28 14:31:43 debug Exp $
+ *  $Id: dev_z8530.c,v 1.16 2007/06/15 19:57:34 debug Exp $
  *  
- *  Zilog "zs" serial controller (Z8530).
+ *  COMMENT: Zilog Z8530 "zs" serial controller
  *
  *  Features:
  *	o)  Two channels, 0 = "channel B", 1 = "channel A".
@@ -95,7 +95,7 @@ static void check_incoming(struct cpu *cpu, struct z8530_data *d)
 DEVICE_TICK(z8530)
 {
 	/*  Generate transmit and receive interrupts at regular intervals.  */
-	struct z8530_data *d = (struct z8530_data *) extra;
+	struct z8530_data *d = extra;
 	int asserted = 0;
 
 	if (d->rr[1][3] & ZSRR3_IP_B_TX && d->wr[0][1] & ZSWR1_TIE)
@@ -203,13 +203,10 @@ DEVICE_ACCESS(z8530)
 
 DEVINIT(z8530)
 {
-	struct z8530_data *d = malloc(sizeof(struct z8530_data));
+	struct z8530_data *d;
 	char tmp[100];
 
-	if (d == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(d = malloc(sizeof(struct z8530_data)));
 	memset(d, 0, sizeof(struct z8530_data));
 
 	d->addr_mult  = devinit->addr_mult;
@@ -233,7 +230,7 @@ DEVINIT(z8530)
 	    NULL);
 
 	machine_add_tickfunction(devinit->machine, dev_z8530_tick, d,
-	    ZS_TICK_SHIFT, 0.0);
+	    ZS_TICK_SHIFT);
 
 	devinit->return_ptr = (void *)(size_t) d->console_handle[0];
 
