@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_zero.c,v 1.6 2006/12/30 13:31:00 debug Exp $
+ *  $Id: dev_zero.c,v 1.8 2007/06/15 20:11:27 debug Exp $
  *  
- *  A simple device which returns zero for all reads, and discards all writes.
+ *  COMMENT: A simple device which returns zero for reads, discards all writes
  */
 
 #include <stdio.h>
@@ -43,7 +43,7 @@
 DEVICE_ACCESS(zero)
 {
 	if (writeflag == MEM_READ) {
-		unsigned int i;
+		size_t i;
 		for (i=0; i<len; i++)
 			data[i] = 0;
 	}
