@@ -2,7 +2,7 @@
 #define	BUS_ISA_H
 
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,18 +28,24 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bus_isa.h,v 1.4 2006/01/16 01:45:50 debug Exp $
+ *  $Id: bus_isa.h,v 1.9 2007/01/17 20:11:28 debug Exp $
  *
  *  ISA bus.
  */
 
 #include "misc.h"
+#include "interrupt.h"
 
 #ifdef BUS_ISA_C
 
 struct bus_isa_data {
-	int		isa_irqbase;
-	int		reassert_irq;
+	struct interrupt irq;
+
+	struct pic8259_data* pic1;
+	struct pic8259_data* pic2;
+	int		*ptr_to_pending_timer_interrupts;
+	int		*ptr_to_last_int;
+
 	uint64_t	isa_portbase;
 	uint64_t	isa_membase;
 };
@@ -47,8 +53,8 @@ struct bus_isa_data {
 #endif	/*  BUS_ISA_C  */
 
 struct bus_isa_data *bus_isa_init(struct machine *machine,
-	uint32_t bus_isa_flags, uint64_t isa_portbase, uint64_t isa_membase,
-	int isa_irqbase, int reassert_irq);
+	char *interrupt_base_path, uint32_t bus_isa_flags,
+	uint64_t isa_portbase, uint64_t isa_membase);
 
 /*  ISA bus flags:  */
 #define	BUS_ISA_IDE0			1
@@ -60,5 +66,6 @@ struct bus_isa_data *bus_isa_init(struct machine *machine,
 #define	BUS_ISA_PCKBC_NONPCSTYLE	64
 #define	BUS_ISA_NO_SECOND_PIC		128
 #define	BUS_ISA_LPTBASE_3BC		256
+#define	BUS_ISA_EXTERNAL_PIC		512
 
 #endif	/*  BUS_ISA_H  */
