@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu.h,v 1.116 2007/04/19 15:18:16 debug Exp $
+ *  $Id: cpu.h,v 1.128 2007/06/14 04:53:14 debug Exp $
  *
  *  CPU-related definitions.
  */
@@ -41,10 +41,13 @@
 /*  This is needed for undefining 'mips', 'ppc' etc. on weird systems:  */
 #include "../../config.h"
 
+#include "timer.h"
+
+
 /*
  *  Dyntrans misc declarations, used throughout the dyntrans code.
  *
- *  Note that there is place for all instruction calls within a page,
+ *  Note that there is space for all instruction calls within a page,
  *  and then 2 more. The first one of these "extra" instruction slots is
  *  the end-of-page slot. It transfers control to the first instruction
  *  slot on the next (virtual) page.
@@ -61,7 +64,8 @@
  *
  *  translations is a tiny bitmap indicating which parts of the page have
  *  actual translations. Bit 0 corresponds to the lowest 1/32th of the page,
- *  bit 1 to the second-lowest 1/32th, and so on.
+ *  bit 1 to the second-lowest 1/32th, and so on. This speeds up page
+ *  invalidations, since only part of the page need to be reset.
  */
 #define DYNTRANS_MISC_DECLARATIONS(arch,ARCH,addrtype)  struct \
 	arch ## _instr_call {					\
@@ -145,13 +149,13 @@
  *  full-size tables can fit in virtual memory on modern hosts (both 32-bit
  *  and 64-bit hosts). :-)
  *
- *  Usage: e.g. VPH32(arm,ARM,uint32_t,uint8_t)
- *           or VPH32(sparc,SPARC,uint64_t,uint16_t)
+ *  Usage: e.g. VPH32(arm,ARM)
+ *           or VPH32(sparc,SPARC)
  *
  *  The vph_tlb_entry entries are cpu dependent tlb entries.
  *
  *  The host_load and host_store entries point to host pages; the phys_addr
- *  entries are uint32_t or uint64_t (emulated physical addresses).
+ *  entries are uint32_t (emulated physical addresses).
  *
  *  phys_page points to translation cache physpages.
  *
@@ -159,21 +163,37 @@
  *  The values in this array are the tlb index plus 1, so a value of, say,
  *  3 means tlb index 2. A value of 0 would mean a tlb index of -1, which
  *  is not a valid index. (I.e. no hit.)
+ *
+ *  The VPH32EXTENDED variant adds an additional postfix to the array
+ *  names. Used so far only for usermode addresses in M88K emulation.
  */
 #define	N_VPH32_ENTRIES		1048576
-#define	VPH32(arch,ARCH,paddrtype,tlbindextype)				\
+#define	VPH32(arch,ARCH)						\
 	unsigned char		*host_load[N_VPH32_ENTRIES];		\
 	unsigned char		*host_store[N_VPH32_ENTRIES];		\
-	paddrtype		phys_addr[N_VPH32_ENTRIES];		\
+	uint32_t		phys_addr[N_VPH32_ENTRIES];		\
 	struct arch ## _tc_physpage  *phys_page[N_VPH32_ENTRIES];	\
-	tlbindextype		vaddr_to_tlbindex[N_VPH32_ENTRIES];
+	uint8_t			vaddr_to_tlbindex[N_VPH32_ENTRIES];
+#define	VPH32_16BITVPHENTRIES(arch,ARCH)				\
+	unsigned char		*host_load[N_VPH32_ENTRIES];		\
+	unsigned char		*host_store[N_VPH32_ENTRIES];		\
+	uint32_t		phys_addr[N_VPH32_ENTRIES];		\
+	struct arch ## _tc_physpage  *phys_page[N_VPH32_ENTRIES];	\
+	uint16_t		vaddr_to_tlbindex[N_VPH32_ENTRIES];
+#define	VPH32EXTENDED(arch,ARCH,ex)					\
+	unsigned char		*host_load_ ## ex[N_VPH32_ENTRIES];	\
+	unsigned char		*host_store_ ## ex[N_VPH32_ENTRIES];	\
+	uint32_t		phys_addr_ ## ex[N_VPH32_ENTRIES];	\
+	struct arch ## _tc_physpage  *phys_page_ ## ex[N_VPH32_ENTRIES];\
+	uint8_t			vaddr_to_tlbindex_ ## ex[N_VPH32_ENTRIES];
+
 
 /*
  *  64-bit dyntrans emulated Virtual -> physical -> host address translation:
  *  -------------------------------------------------------------------------
  *
- *  Usage: e.g. VPH64(alpha,ALPHA,uint8_t)
- *           or VPH64(sparc,SPARC,uint16_t)
+ *  Usage: e.g. VPH64(alpha,ALPHA)
+ *           or VPH64(sparc,SPARC)
  *
  *  l1_64 is an array containing poiners to l2 tables.
  *
@@ -182,7 +202,7 @@
  *  used.
  */
 #define	DYNTRANS_L1N		17
-#define	VPH64(arch,ARCH,tlbindextype)					\
+#define	VPH64(arch,ARCH)						\
 	struct arch ## _l3_64_table	*l3_64_dummy;			\
 	struct arch ## _l3_64_table	*next_free_l3;			\
 	struct arch ## _l2_64_table	*l2_64_dummy;			\
@@ -193,7 +213,6 @@
 /*  Include all CPUs' header files here:  */
 #include "cpu_alpha.h"
 #include "cpu_arm.h"
-#include "cpu_avr.h"
 #include "cpu_m88k.h"
 #include "cpu_mips.h"
 #include "cpu_ppc.h"
@@ -274,20 +293,16 @@ struct cpu_family {
 #define	N_SAFE_DYNTRANS_LIMIT_SHIFT	14
 #define	N_SAFE_DYNTRANS_LIMIT	((1 << (N_SAFE_DYNTRANS_LIMIT_SHIFT - 1)) - 1)
 
-#define	DEFAULT_DYNTRANS_CACHE_SIZE	(40*1048576)
+#define	MAX_DYNTRANS_READAHEAD		1024
+
+#define	DEFAULT_DYNTRANS_CACHE_SIZE	(48*1048576)
 #define	DYNTRANS_CACHE_MARGIN		200000
 
 #define	N_BASE_TABLE_ENTRIES		65536
 #define	PAGENR_TO_TABLE_INDEX(a)	((a) & (N_BASE_TABLE_ENTRIES-1))
 
-
-#ifdef NATIVE_CODE_GENERATION
-/*
- *  Intermediate Native Representation (INR).
- *  Used for native code generation.
- */
-#include "inr.h"
-#endif
+#define	CPU_SAMPLE_TIMER_HZ		TIMER_BASE_FREQUENCY
+#define	N_PADDR_SAMPLES			64
 
 
 /*
@@ -307,18 +322,40 @@ struct cpu {
 	/*  Full "path" to the CPU, e.g. "emul[0].machine[0].cpu[0]":  */
 	char		*path;
 
+	/*  Nr of instructions executed, etc.:  */
+	int64_t		ninstrs;
+	int64_t		ninstrs_show;
+	int64_t		ninstrs_flush;
+	int64_t		ninstrs_since_gettimeofday;
+	struct timeval	starttime;
+
+	/*
+	 *  Periodic sampling of the physical address corresponding to the
+	 *  emulated program counter:
+	 *
+	 *  (Used to decide whether or not native code generation is worth
+	 *  the effort.)
+	 */
+	struct timer	*sampling_timer;
+	uint8_t		sampling;	/*  1 = turned on  */
+	int16_t		sampling_curindex;
+	uint64_t	*sampling_paddr;
+
 	/*  EMUL_LITTLE_ENDIAN or EMUL_BIG_ENDIAN.  */
-	int		byte_order;
+	uint8_t		byte_order;
+
+	/*  0 for emulated 64-bit CPUs, 1 for 32-bit.  */
+	uint8_t		is_32bit;
+
+	/*  1 while running, 0 when paused/stopped.  */
+	uint8_t		running;
+
+	/*  See comment further up.  */
+	uint8_t		delay_slot;
 
 	/*  0-based CPU id, in an emulated SMP system.  */
 	int		cpu_id;
 
-	/*  0 for emulated 64-bit CPUs, 1 for 32-bit.  */
-	int		is_32bit;
-
-	/*  1 while running, 0 when paused/stopped.  */
-	int		running;
-
 	/*  A pointer to the main memory connected to this CPU.  */
 	struct memory	*mem;
 
@@ -343,9 +380,6 @@ struct cpu {
 	/*  The program counter. (For 32-bit modes, not all bits are used.)  */
 	uint64_t	pc;
 
-	/*  See comment further up.  */
-	int		delay_slot;
-
 	/*  The current depth of function call tracing.  */
 	int		trace_tree_depth;
 
@@ -358,8 +392,8 @@ struct cpu {
 	 *  instructions per second, "idling" is printed instead. (The number
 	 *  of instrs per second when idling is meaningless anyway.)
 	 */
-	int		is_halted;
-	int		has_been_idling;
+	char		is_halted;
+	char		has_been_idling;
 
 	/*
 	 *  Dynamic translation:
@@ -381,27 +415,29 @@ struct cpu {
 	 *  32 MB) which is used for translations. When it has been used up,
 	 *  everything restarts from scratch.
 	 *
-	 *  The INR struct contains the Intermediate Native Representation,
-	 *  used during native code generation.
+	 *  translation_readahead is non-zero when translating instructions
+	 *  ahead of the current (emulated) instruction pointer.
 	 */
+
+	/*  Non-zero when translating ahead of the current instruction:  */
+	int		translation_readahead;
+
+	/*  Instruction translation cache:  */
 	int		n_translated_instrs;
 	unsigned char	*translation_cache;
 	size_t		translation_cache_cur_ofs;
 
-#ifdef NATIVE_CODE_GENERATION
-	struct inr	inr;
-#endif
 
 	/*
 	 *  CPU-family dependent:
 	 *
-	 *  These contain everything ranging from registers, memory management,
-	 *  status words, etc.
+	 *  These contain everything ranging from general purpose registers,
+	 *  control registers, memory management, status words, interrupt
+	 *  specifics, etc.
 	 */
 	union {
 		struct alpha_cpu      alpha;
 		struct arm_cpu        arm;
-		struct avr_cpu        avr;
 		struct m88k_cpu       m88k;
 		struct mips_cpu       mips;
 		struct ppc_cpu        ppc;
