@@ -28,13 +28,15 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_alpha.h,v 1.45 2006/12/30 13:31:00 debug Exp $
+ *  $Id: cpu_alpha.h,v 1.48 2007/06/07 15:36:24 debug Exp $
  */
 
 #include "misc.h"
 
 #include "alpha_cpu.h"
 
+struct timer;
+
 
 /*  ALPHA CPU types:  */
 struct alpha_cpu_type_def { 
@@ -134,6 +136,8 @@ struct alpha_cpu {
 	uint64_t		load_linked_addr;
 	int			ll_flag;
 
+	int			irq_asserted;
+
 	/*  OSF1 PALcode specific:  */
 	uint64_t		vptptr;		/*  Virtual Page Table Ptr  */
 	uint64_t		sysvalue;
@@ -149,7 +153,7 @@ struct alpha_cpu {
 	 */
 	DYNTRANS_ITC(alpha)
 	VPH_TLBS(alpha,ALPHA)
-	VPH64(alpha,ALPHA,uint8_t)
+	VPH64(alpha,ALPHA)
 };
 
 
@@ -159,6 +163,7 @@ void alpha_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 void alpha_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void alpha_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 void alpha_init_64bit_dummy_tables(struct cpu *cpu);
+void alpha_timer_sample_tick(struct timer *, void *);
 int alpha_run_instr(struct cpu *cpu);
 int alpha_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
