@@ -28,13 +28,14 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_arm.h,v 1.70 2007/02/05 16:49:21 debug Exp $
+ *  $Id: cpu_arm.h,v 1.74 2007/06/14 04:53:14 debug Exp $
  */
 
 #include "misc.h"
 #include "interrupt.h"
 
 struct cpu_family;
+struct timer;
 
 /*  ARM CPU types:  */
 struct arm_cpu_type_def {
@@ -69,11 +70,7 @@ struct arm_cpu_type_def {
 	"and", "eor", "sub", "rsb", "add", "adc", "sbc", "rsc",	\
 	"tst", "teq", "cmp", "cmn", "orr", "mov", "bic", "mvn" }
 
-#ifdef ONEKPAGE
-#define	ARM_IC_ENTRIES_SHIFT		8
-#else
 #define	ARM_IC_ENTRIES_SHIFT		10
-#endif
 
 #define	ARM_N_IC_ARGS			3
 #define	ARM_INSTR_ALIGNMENT_SHIFT	2
@@ -127,7 +124,7 @@ struct arm_cpu_type_def {
 
 DYNTRANS_MISC_DECLARATIONS(arm,ARM,uint32_t)
 
-#define	ARM_MAX_VPH_TLB_ENTRIES		128
+#define	ARM_MAX_VPH_TLB_ENTRIES		384
 
 
 struct arm_cpu {
@@ -237,7 +234,7 @@ struct arm_cpu {
 	 */
 	DYNTRANS_ITC(arm)
 	VPH_TLBS(arm,ARM)
-	VPH32(arm,ARM,uint32_t,uint8_t)
+	VPH32_16BITVPHENTRIES(arm,ARM)
 
 	/*  ARM specific: */
 	uint32_t			is_userpage[N_VPH32_ENTRIES/32];
@@ -293,6 +290,7 @@ void arm_translation_table_set_l1_b(struct cpu *cpu, uint32_t vaddr,
 	uint32_t paddr);
 void arm_exception(struct cpu *, int);
 int arm_run_instr(struct cpu *cpu);
+void arm_timer_sample_tick(struct timer *, void *);
 void arm_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
 void arm_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
