@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips.h,v 1.56 2007/04/28 09:19:52 debug Exp $
+ *  $Id: cpu_mips.h,v 1.59 2007/06/07 15:36:25 debug Exp $
  */
 
 #include "interrupt.h"
@@ -182,16 +182,8 @@ struct r3000_cache_line {
 #define	R3000_TAG_VALID		1
 #define	R3000_TAG_DIRTY		2
 
-struct r4000_cache_line {
-	char		dummy;
-};
 
-
-#ifdef ONEKPAGE
-#define	MIPS_IC_ENTRIES_SHIFT		8
-#else
 #define	MIPS_IC_ENTRIES_SHIFT		10
-#endif
 
 #define	MIPS_N_IC_ARGS			3
 #define	MIPS_INSTR_ALIGNMENT_SHIFT	2
@@ -205,6 +197,7 @@ struct r4000_cache_line {
 #define	MIPS_L3N		18
 
 #define	MIPS_MAX_VPH_TLB_ENTRIES	128
+
 DYNTRANS_MISC_DECLARATIONS(mips,MIPS,uint64_t)
 DYNTRANS_MISC64_DECLARATIONS(mips,MIPS,uint8_t)
 
@@ -255,14 +248,26 @@ struct mips_cpu {
 	uint64_t	lo1;
 	uint32_t	r5900_sa;
 
-	/*  Data and Instruction caches:  */
+
+	/*
+	 *  Data and Instruction caches:
+	 */
+
+	/*  Cache sizes: (1 << x) x=0 for default values  */
+	/*  This is legacy stuff. TODO: Clean up!  */
+	int		cache_picache;
+	int		cache_pdcache;
+	int		cache_secondary;
+	int		cache_picache_linesize;
+	int		cache_pdcache_linesize;
+	int		cache_secondary_linesize;
+
 	unsigned char	*cache[2];
 	void		*cache_tags[2];
 	uint64_t	cache_last_paddr[2];
 	int		cache_size[2];
 	int		cache_linesize[2];
 	int		cache_mask[2];
-	int		cache_miss_penalty[2];
 
 
 	/*
@@ -271,8 +276,8 @@ struct mips_cpu {
 	 */
 	DYNTRANS_ITC(mips)
 	VPH_TLBS(mips,MIPS)
-	VPH32(mips,MIPS,uint64_t,uint8_t)
-	VPH64(mips,MIPS,uint8_t)
+	VPH32(mips,MIPS)
+	VPH64(mips,MIPS)
 };
 
 
@@ -338,6 +343,7 @@ void mips_unaligned_loadstore(struct cpu *cpu, struct mips_instr_call *ic,
 
 
 int mips_run_instr(struct cpu *cpu);
+void mips_timer_sample_tick(struct timer *, void *);
 void mips_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
 void mips_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
