@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sh.h,v 1.41 2007/04/28 09:19:34 debug Exp $
+ *  $Id: cpu_sh.h,v 1.46 2007/06/07 15:36:25 debug Exp $
  *
  *  Note: Many things here are SH4-specific, so it probably doesn't work
  *        for SH3 emulation.
@@ -40,6 +40,8 @@
 
 
 struct cpu_family;
+struct timer;
+
 
 /*  SH CPU types:  */
 struct sh_cpu_type_def {
@@ -51,10 +53,11 @@ struct sh_cpu_type_def {
 };
 
 #define	SH_CPU_TYPE_DEFS				{	    \
+	{ "SH7708R", 32, 3, 0,              0,                   }, \
 	{ "SH7750",  32, 4, SH4_PVR_SH7750, 0	                 }, \
 	{ "SH7750R", 32, 4, SH4_PVR_SH7750, SH4_PRR_7750R	 }, \
 	{ "SH7751R", 32, 4, SH4_PVR_SH7751, SH4_PRR_7751R	 }, \
-	{ "SH5",     64, 5, 0,              0	                 }, \
+	/* { "SH5",  64, 5, 0,              0	                 }, */ \
 	{ NULL,       0, 0, 0,              0	                 }  }
 
 
@@ -90,9 +93,6 @@ DYNTRANS_MISC_DECLARATIONS(sh,SH,uint32_t)
 struct sh_cpu {
 	struct sh_cpu_type_def cpu_type;
 
-	/*  compact = 1 if currently executing 16-bit long opcodes  */
-	int		compact;
-
 	/*  General Purpose Registers:  */
 	uint32_t	r[SH_N_GPRS];
 	uint32_t	r_bank[SH_N_GPRS_BANKED];
@@ -173,7 +173,7 @@ struct sh_cpu {
 	 */
 	DYNTRANS_ITC(sh)
 	VPH_TLBS(sh,SH)
-	VPH32(sh,SH,uint32_t,uint8_t)
+	VPH32(sh,SH)
 };
 
 
@@ -234,6 +234,7 @@ void sh32_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 void sh32_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void sh32_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 void sh_init_64bit_dummy_tables(struct cpu *cpu);
+void sh_timer_sample_tick(struct timer *, void *);
 int sh_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
 int sh_cpu_family_init(struct cpu_family *);
