@@ -28,13 +28,14 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sparc.h,v 1.46 2007/03/16 18:49:06 debug Exp $
+ *  $Id: cpu_sparc.h,v 1.48 2007/06/07 15:36:25 debug Exp $
  */
 
 #include "misc.h"
 
 
 struct cpu_family;
+struct timer;
 
 
 /*  SPARC CPU types:  */
@@ -248,8 +249,8 @@ struct sparc_cpu {
 	 */
 	DYNTRANS_ITC(sparc)
 	VPH_TLBS(sparc,SPARC)
-	VPH32(sparc,SPARC,uint64_t,uint8_t)
-	VPH64(sparc,SPARC,uint8_t)
+	VPH32(sparc,SPARC)
+	VPH64(sparc,SPARC)
 };
 
 
@@ -349,6 +350,7 @@ void sparc32_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 void sparc32_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void sparc32_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 void sparc_init_64bit_dummy_tables(struct cpu *cpu);
+void sparc_timer_sample_tick(struct timer *, void *);
 int sparc_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
 int sparc_cpu_family_init(struct cpu_family *);
