@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: misc.h,v 1.251 2007/03/16 14:45:30 debug Exp $
+ *  $Id: misc.h,v 1.257 2007/06/15 17:12:31 debug Exp $
  *
  *  Misc. definitions for gxemul.
  */
@@ -162,6 +162,36 @@ struct memory;
 #define	DEBUG_INDENTATION	4
 
 
+#ifdef HAVE___FUNCTION__
+
+#define	FAILURE(error_msg)					{	\
+		char where_msg[400];					\
+		snprintf(where_msg, sizeof(where_msg),			\
+		    "%s, line %i, function '%s'\n",			\
+		    __FILE__, __LINE__, __FUNCTION__);			\
+        	fprintf(stderr, "%s: %s\n", error_msg, where_msg);	\
+		exit(1);						\
+	}
+
+#else
+
+#define	FAILURE(error_msg)					{	\
+		char where_msg[400];					\
+		snprintf(where_msg, sizeof(where_msg),			\
+		    "%s, line %i\n", __FILE__, __LINE__);		\
+        	fprintf(stderr, "%s: %s\n", error_msg, where_msg);	\
+		exit(1);						\
+	}
+
+#endif	/*  !HAVE___FUNCTION__  */
+
+
+#define	CHECK_ALLOCATION(ptr)					{	\
+		if ((ptr) == NULL)					\
+			FAILURE("Out of memory");			\
+	}
+
+
 /*  bootblock.c:  */
 int load_bootblock(struct machine *m, struct cpu *cpu,
 	int *n_loadp, char ***load_namesp);
@@ -211,12 +241,12 @@ int mymkstemp(char *template);
 size_t mystrlcpy(char *dst, const char *src, size_t size);
 size_t mystrlcat(char *dst, const char *src, size_t size);
 #endif
+void print_separator_line(void);
 
 
-/*  pc_bios.c:  */
-void pc_bios_simple_pmode_setup(struct cpu *cpu);
-void pc_bios_init(struct cpu *cpu);
-int pc_bios_emul(struct cpu *cpu);
+/*  mvmeprom.c:  */
+void mvmeprom_init(struct machine *machine);
+int mvmeprom_emul(struct cpu *cpu);
 
 
 /*  ps2_bios.c:  */
@@ -228,15 +258,6 @@ void sh_ipl_g_emul_init(struct machine *machine);
 int sh_ipl_g_emul(struct cpu *);
 
 
-/*  useremul.c:  */
-void useremul_setup(struct cpu *, int, char **);
-void useremul_syscall(struct cpu *cpu, uint32_t code);
-void useremul_name_to_useremul(struct cpu *, char *name,
-	int *arch, char **machine_name, char **cpu_name);
-void useremul_list_emuls(void);
-void useremul_init(void);
-
-
 /*  yamon.c:  */
 void yamon_machine_setup(struct machine *machine, uint64_t env);
 int yamon_emul(struct cpu *);
