@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: machine.c,v 1.693 2007/01/28 14:15:29 debug Exp $
+ *  $Id: machine.c,v 1.704 2007/06/15 17:02:38 debug Exp $
  */
 
 #include <stdio.h>
@@ -35,21 +35,16 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "arcbios.h"
-#include "bus_isa.h"
-#include "bus_pci.h"
 #include "cpu.h"
-#include "debugger.h"
 #include "device.h"
-#include "devices.h"
 #include "diskimage.h"
 #include "emul.h"
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "net.h"
 #include "settings.h"
 #include "symbol.h"
+#include "useremul.h"
 
 
 /*  See main.c:  */
@@ -69,25 +64,17 @@ struct machine_entry *first_machine_entry = NULL;
 struct machine *machine_new(char *name, struct emul *emul, int id)
 {
 	struct machine *m;
-	m = malloc(sizeof(struct machine));
-	if (m == NULL) {
-		fprintf(stderr, "machine_new(): out of memory\n");
-		exit(1);
-	}
 
+	CHECK_ALLOCATION(m = malloc(sizeof(struct machine)));
 	memset(m, 0, sizeof(struct machine));
 
 	/*  Pointer back to the emul object that this machine belongs to:  */
 	m->emul = emul;
 
-	m->name = strdup(name);
+	CHECK_ALLOCATION(m->name = strdup(name));
 
 	/*  Full path, e.g. "emul[0].machine[0]":  */
-	m->path = malloc(strlen(emul->path) + 20);
-	if (m->path == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(m->path = malloc(strlen(emul->path) + 20));
 	snprintf(m->path, strlen(emul->path) + 20, "%s.machine[%i]",
 	    emul->path, id);
 
@@ -102,11 +89,9 @@ struct machine *machine_new(char *name, struct emul *emul, int id)
 	m->byte_order_override = NO_BYTE_ORDER_OVERRIDE;
 	m->boot_kernel_filename = "";
 	m->boot_string_argument = NULL;
-	m->x11_scaledown = 1;
-	m->x11_scaleup = 1;
+	m->x11_md.scaledown = 1;
+	m->x11_md.scaleup = 1;
 	m->n_gfx_cards = 1;
-	m->dbe_on_nonexistant_memaccess = 1;
-	m->show_symbolic_register_names = 1;
 	symbol_init(&m->symbol_context);
 
 	/*  Settings:  */
@@ -129,9 +114,6 @@ struct machine *machine_new(char *name, struct emul *emul, int id)
 	settings_add(m->settings, "n_gfx_cards", 0,
 	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_DECIMAL,
 	    (void *) &m->n_gfx_cards);
-	settings_add(m->settings, "show_symbolic_register_names", 1,
-	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
-	    (void *) &m->show_symbolic_register_names);
 	settings_add(m->settings, "statistics_enabled", 1,
 	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
 	    (void *) &m->statistics_enabled);
@@ -265,6 +247,28 @@ int machine_name_to_type(char *stype, char *ssubtype,
 }
 
 
+/*
+ *  machine_add_breakpoint_string():
+ *
+ *  Add a breakpoint string to the machine. Later (in emul.c) these will be
+ *  converted to actual breakpoints.
+ */
+void machine_add_breakpoint_string(struct machine *machine, char *str)
+{
+	int n = machine->breakpoints.n + 1;
+
+	CHECK_ALLOCATION(machine->breakpoints.string =
+	    realloc(machine->breakpoints.string, n * sizeof(char *)));
+	CHECK_ALLOCATION(machine->breakpoints.addr =
+	    realloc(machine->breakpoints.addr, n * sizeof(uint64_t)));
+	CHECK_ALLOCATION(machine->breakpoints.string[machine->breakpoints.n] =
+	    strdup(optarg));
+
+	machine->breakpoints.addr[machine->breakpoints.n] = 0;
+	machine->breakpoints.n ++;
+}
+
+
 /*
  *  machine_add_tickfunction():
  *
@@ -278,37 +282,37 @@ int machine_name_to_type(char *stype, char *ssubtype,
  *  The hz value is used in this case.
  */
 void machine_add_tickfunction(struct machine *machine, void (*func)
-	(struct cpu *, void *), void *extra, int tickshift, double hz)
+	(struct cpu *, void *), void *extra, int tickshift)
 {
-	int n = machine->n_tick_entries;
+	int n = machine->tick_functions.n_entries;
 
-	if (n >= MAX_TICK_FUNCTIONS) {
-		fprintf(stderr, "machine_add_tickfunction(): too "
-		    "many tick functions\n");
+	CHECK_ALLOCATION(machine->tick_functions.ticks_till_next = realloc(
+	    machine->tick_functions.ticks_till_next, (n+1) * sizeof(int)));
+	CHECK_ALLOCATION(machine->tick_functions.ticks_reset_value = realloc(
+	    machine->tick_functions.ticks_reset_value, (n+1) * sizeof(int)));
+	CHECK_ALLOCATION(machine->tick_functions.f = realloc(
+	    machine->tick_functions.f, (n+1) * sizeof(void *)));
+	CHECK_ALLOCATION(machine->tick_functions.extra = realloc(
+	    machine->tick_functions.extra, (n+1) * sizeof(void *)));
+
+	/*
+	 *  The dyntrans subsystem wants to run code in relatively
+	 *  large chunks without checking for external interrupts;
+	 *  too low tickshifts are not allowed.
+	 */
+	if (tickshift < N_SAFE_DYNTRANS_LIMIT_SHIFT) {
+		fatal("ERROR! tickshift = %i, less than "
+		    "N_SAFE_DYNTRANS_LIMIT_SHIFT (%i)\n",
+		    tickshift, N_SAFE_DYNTRANS_LIMIT_SHIFT);
 		exit(1);
 	}
 
-	if (!machine->cycle_accurate) {
-		/*
-		 *  The dyntrans subsystem wants to run code in relatively
-		 *  large chunks without checking for external interrupts;
-		 *  too low tickshifts are not allowed.
-		 */
-		if (tickshift < N_SAFE_DYNTRANS_LIMIT_SHIFT) {
-			fatal("ERROR! tickshift = %i, less than "
-			    "N_SAFE_DYNTRANS_LIMIT_SHIFT (%i)\n",
-			    tickshift, N_SAFE_DYNTRANS_LIMIT_SHIFT);
-			exit(1);
-		}
-	}
+	machine->tick_functions.ticks_till_next[n]   = 0;
+	machine->tick_functions.ticks_reset_value[n] = 1 << tickshift;
+	machine->tick_functions.f[n]                 = func;
+	machine->tick_functions.extra[n]             = extra;
 
-	machine->ticks_till_next[n]   = 0;
-	machine->ticks_reset_value[n] = 1 << tickshift;
-	machine->tick_func[n]         = func;
-	machine->tick_extra[n]        = extra;
-	machine->tick_hz[n]           = hz;
-
-	machine->n_tick_entries ++;
+	machine->tick_functions.n_entries = n + 1;
 }
 
 
@@ -333,8 +337,9 @@ void machine_statistics_init(struct machine *machine, char *fname)
 		exit(1);
 	}
 
-	machine->statistics_fields = malloc(MAX_STATISTICS_FIELDS + 1);
 	machine->statistics_enabled = 1;
+	CHECK_ALLOCATION(machine->statistics_fields = malloc(1));
+	machine->statistics_fields[0] = '\0';
 
 	while (*pcolon && *pcolon != ':')
 		pcolon ++;
@@ -353,13 +358,10 @@ void machine_statistics_init(struct machine *machine, char *fname)
 		case 'v':
 		case 'i':
 		case 'p':
+			CHECK_ALLOCATION(machine->statistics_fields = realloc(
+			    machine->statistics_fields, strlen(
+			    machine->statistics_fields) + 2));
 			machine->statistics_fields[n_fields ++] = *fname;
-			if (n_fields >= MAX_STATISTICS_FIELDS) {
-				fprintf(stderr, "Internal error: Too many "
-				    "statistics fields used. Increase "
-				    "MAX_STATISTICS_FIELDS.\n");
-				exit(1);
-			}
 			machine->statistics_fields[n_fields] = '\0';
 			break;
 
@@ -380,7 +382,7 @@ void machine_statistics_init(struct machine *machine, char *fname)
 
 	fname ++;	/*  point to the filename after the colon  */
 
-	machine->statistics_filename = strdup(fname);
+	CHECK_ALLOCATION(machine->statistics_filename = strdup(fname));
 	machine->statistics_file = fopen(machine->statistics_filename, mode);
 }
 
@@ -405,8 +407,6 @@ void machine_dumpinfo(struct machine *m)
 		debug(" (offset by %i MB)", m->memory_offset_in_mb);
 	if (m->random_mem_contents)
 		debug(", randomized contents");
-	if (m->dbe_on_nonexistant_memaccess)
-		debug(", dbe_on_nonexistant_memaccess");
 	debug("\n");
 
 	if (!m->prom_emulation)
@@ -421,16 +421,16 @@ void machine_dumpinfo(struct machine *m)
 	if (m->slow_serial_interrupts_hack_for_linux)
 		debug("Using slow_serial_interrupts_hack_for_linux\n");
 
-	if (m->use_x11) {
+	if (m->x11_md.in_use) {
 		debug("Using X11");
-		if (m->x11_scaledown > 1)
-			debug(", scaledown %i", m->x11_scaledown);
-		if (m->x11_scaleup > 1)
-			debug(", scaleup %i", m->x11_scaleup);
-		if (m->x11_n_display_names > 0) {
-			for (i=0; i<m->x11_n_display_names; i++) {
+		if (m->x11_md.scaledown > 1)
+			debug(", scaledown %i", m->x11_md.scaledown);
+		if (m->x11_md.scaleup > 1)
+			debug(", scaleup %i", m->x11_md.scaleup);
+		if (m->x11_md.n_display_names > 0) {
+			for (i=0; i<m->x11_md.n_display_names; i++) {
 				debug(i? ", " : " (");
-				debug("\"%s\"", m->x11_display_names[i]);
+				debug("\"%s\"", m->x11_md.display_names[i]);
 			}
 			debug(")");
 		}
@@ -444,382 +444,6 @@ void machine_dumpinfo(struct machine *m)
 }
 
 
-/*
- *  dump_mem_string():
- *
- *  Dump the contents of emulated RAM as readable text.  Bytes that aren't
- *  readable are dumped in [xx] notation, where xx is in hexadecimal.
- *  Dumping ends after DUMP_MEM_STRING_MAX bytes, or when a terminating
- *  zero byte is found.
- */
-#define DUMP_MEM_STRING_MAX	45
-void dump_mem_string(struct cpu *cpu, uint64_t addr)
-{
-	int i;
-	for (i=0; i<DUMP_MEM_STRING_MAX; i++) {
-		unsigned char ch = '\0';
-
-		cpu->memory_rw(cpu, cpu->mem, addr + i, &ch, sizeof(ch),
-		    MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
-		if (ch == '\0')
-			return;
-		if (ch >= ' ' && ch < 126)
-			debug("%c", ch);  
-		else
-			debug("[%02x]", ch);
-	}
-}
-
-
-/*
- *  store_byte():
- *
- *  Stores a byte in emulated ram. (Helper function.)
- */
-void store_byte(struct cpu *cpu, uint64_t addr, uint8_t data)
-{
-	if ((addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-	cpu->memory_rw(cpu, cpu->mem,
-	    addr, &data, sizeof(data), MEM_WRITE, CACHE_DATA);
-}
-
-
-/*
- *  store_string():
- *
- *  Stores chars into emulated RAM until a zero byte (string terminating
- *  character) is found. The zero byte is also copied.
- *  (strcpy()-like helper function, host-RAM-to-emulated-RAM.)
- */
-void store_string(struct cpu *cpu, uint64_t addr, char *s)
-{
-	do {
-		store_byte(cpu, addr++, *s);
-	} while (*s++);
-}
-
-
-/*
- *  add_environment_string():
- *
- *  Like store_string(), but advances the pointer afterwards. The most
- *  obvious use is to place a number of strings (such as environment variable
- *  strings) after one-another in emulated memory.
- */
-void add_environment_string(struct cpu *cpu, char *s, uint64_t *addr)
-{
-	store_string(cpu, *addr, s);
-	(*addr) += strlen(s) + 1;
-}
-
-
-/*
- *  add_environment_string_dual():
- *
- *  Add "dual" environment strings, one for the variable name and one for the
- *  value, and update pointers afterwards.
- */
-void add_environment_string_dual(struct cpu *cpu,
-	uint64_t *ptrp, uint64_t *addrp, char *s1, char *s2)
-{
-	uint64_t ptr = *ptrp, addr = *addrp;
-
-	store_32bit_word(cpu, ptr, addr);
-	ptr += sizeof(uint32_t);
-	if (addr != 0) {
-		store_string(cpu, addr, s1);
-		addr += strlen(s1) + 1;
-	}
-	store_32bit_word(cpu, ptr, addr);
-	ptr += sizeof(uint32_t);
-	if (addr != 0) {
-		store_string(cpu, addr, s2);
-		addr += strlen(s2) + 1;
-	}
-
-	*ptrp = ptr;
-	*addrp = addr;
-}
-
-
-/*
- *  store_64bit_word():
- *
- *  Stores a 64-bit word in emulated RAM.  Byte order is taken into account.
- *  Helper function.
- */
-int store_64bit_word(struct cpu *cpu, uint64_t addr, uint64_t data64)
-{
-	unsigned char data[8];
-	if ((addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-	data[0] = (data64 >> 56) & 255;
-	data[1] = (data64 >> 48) & 255;
-	data[2] = (data64 >> 40) & 255;
-	data[3] = (data64 >> 32) & 255;
-	data[4] = (data64 >> 24) & 255;
-	data[5] = (data64 >> 16) & 255;
-	data[6] = (data64 >> 8) & 255;
-	data[7] = (data64) & 255;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[7]; data[7] = tmp;
-		tmp = data[1]; data[1] = data[6]; data[6] = tmp;
-		tmp = data[2]; data[2] = data[5]; data[5] = tmp;
-		tmp = data[3]; data[3] = data[4]; data[4] = tmp;
-	}
-	return cpu->memory_rw(cpu, cpu->mem,
-	    addr, data, sizeof(data), MEM_WRITE, CACHE_DATA);
-}
-
-
-/*
- *  store_32bit_word():
- *
- *  Stores a 32-bit word in emulated RAM.  Byte order is taken into account.
- *  (This function takes a 64-bit word as argument, to suppress some
- *  warnings, but only the lowest 32 bits are used.)
- */
-int store_32bit_word(struct cpu *cpu, uint64_t addr, uint64_t data32)
-{
-	unsigned char data[4];
-
-	/*  TODO: REMOVE THIS once everything is more stable!  */
-	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-
-	data[0] = (data32 >> 24) & 255;
-	data[1] = (data32 >> 16) & 255;
-	data[2] = (data32 >> 8) & 255;
-	data[3] = (data32) & 255;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[3]; data[3] = tmp;
-		tmp = data[1]; data[1] = data[2]; data[2] = tmp;
-	}
-	return cpu->memory_rw(cpu, cpu->mem,
-	    addr, data, sizeof(data), MEM_WRITE, CACHE_DATA);
-}
-
-
-/*
- *  store_16bit_word():
- *
- *  Stores a 16-bit word in emulated RAM.  Byte order is taken into account.
- *  (This function takes a 64-bit word as argument, to suppress some
- *  warnings, but only the lowest 16 bits are used.)
- */
-int store_16bit_word(struct cpu *cpu, uint64_t addr, uint64_t data16)
-{
-	unsigned char data[2];
-
-	/*  TODO: REMOVE THIS once everything is more stable!  */
-	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-
-	data[0] = (data16 >> 8) & 255;
-	data[1] = (data16) & 255;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[1]; data[1] = tmp;
-	}
-	return cpu->memory_rw(cpu, cpu->mem,
-	    addr, data, sizeof(data), MEM_WRITE, CACHE_DATA);
-}
-
-
-/*
- *  store_buf():
- *
- *  memcpy()-like helper function, from host RAM to emulated RAM.
- */
-void store_buf(struct cpu *cpu, uint64_t addr, char *s, size_t len)
-{
-	size_t psize = 1024;	/*  1024 256 64 16 4 1  */
-
-	/*  TODO: REMOVE THIS once everything is more stable!  */
-	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-
-	while (len != 0) {
-		if ((addr & (psize-1)) == 0) {
-			while (len >= psize) {
-				cpu->memory_rw(cpu, cpu->mem, addr,
-				    (unsigned char *)s, psize, MEM_WRITE,
-				    CACHE_DATA);
-				addr += psize;
-				s += psize;
-				len -= psize;
-			}
-		}
-		psize >>= 2;
-	}
-
-	while (len-- != 0)
-		store_byte(cpu, addr++, *s++);
-}
-
-
-/*
- *  store_pointer_and_advance():
- *
- *  Stores a 32-bit or 64-bit pointer in emulated RAM, and advances the
- *  target address. (Useful for e.g. ARCBIOS environment initialization.)
- */
-void store_pointer_and_advance(struct cpu *cpu, uint64_t *addrp,
-	uint64_t data, int flag64)
-{
-	uint64_t addr = *addrp;
-	if (flag64) {
-		store_64bit_word(cpu, addr, data);
-		addr += 8;
-	} else {
-		store_32bit_word(cpu, addr, data);
-		addr += 4;
-	}
-	*addrp = addr;
-}
-
-
-/*
- *  load_64bit_word():
- *
- *  Helper function. Emulated byte order is taken into account.
- */
-uint64_t load_64bit_word(struct cpu *cpu, uint64_t addr)
-{
-	unsigned char data[8];
-
-	cpu->memory_rw(cpu, cpu->mem,
-	    addr, data, sizeof(data), MEM_READ, CACHE_DATA);
-
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[7]; data[7] = tmp;
-		tmp = data[1]; data[1] = data[6]; data[6] = tmp;
-		tmp = data[2]; data[2] = data[5]; data[5] = tmp;
-		tmp = data[3]; data[3] = data[4]; data[4] = tmp;
-	}
-
-	return
-	    ((uint64_t)data[0] << 56) + ((uint64_t)data[1] << 48) +
-	    ((uint64_t)data[2] << 40) + ((uint64_t)data[3] << 32) +
-	    ((uint64_t)data[4] << 24) + ((uint64_t)data[5] << 16) +
-	    ((uint64_t)data[6] << 8) + (uint64_t)data[7];
-}
-
-
-/*
- *  load_32bit_word():
- *
- *  Helper function. Emulated byte order is taken into account.
- */
-uint32_t load_32bit_word(struct cpu *cpu, uint64_t addr)
-{
-	unsigned char data[4];
-
-	/*  TODO: REMOVE THIS once everything is more stable!  */
-	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-
-	cpu->memory_rw(cpu, cpu->mem,
-	    addr, data, sizeof(data), MEM_READ, CACHE_DATA);
-
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[3]; data[3] = tmp;
-		tmp = data[1]; data[1] = data[2]; data[2] = tmp;
-	}
-
-	return (data[0] << 24) + (data[1] << 16) + (data[2] << 8) + data[3];
-}
-
-
-/*
- *  load_16bit_word():
- *
- *  Helper function. Emulated byte order is taken into account.
- */
-uint16_t load_16bit_word(struct cpu *cpu, uint64_t addr)
-{
-	unsigned char data[2];
-
-	/*  TODO: REMOVE THIS once everything is more stable!  */
-	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
-		addr = (int64_t)(int32_t)addr;
-
-	cpu->memory_rw(cpu, cpu->mem,
-	    addr, data, sizeof(data), MEM_READ, CACHE_DATA);
-
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[1]; data[1] = tmp;
-	}
-
-	return (data[0] << 8) + data[1];
-}
-
-
-/*
- *  store_64bit_word_in_host():
- *
- *  Stores a 64-bit word in the _host's_ RAM.  Emulated byte order is taken
- *  into account.  This is useful when building structs in the host's RAM
- *  which will later be copied into emulated RAM.
- */
-void store_64bit_word_in_host(struct cpu *cpu,
-	unsigned char *data, uint64_t data64)
-{
-	data[0] = (data64 >> 56) & 255;
-	data[1] = (data64 >> 48) & 255;
-	data[2] = (data64 >> 40) & 255;
-	data[3] = (data64 >> 32) & 255;
-	data[4] = (data64 >> 24) & 255;
-	data[5] = (data64 >> 16) & 255;
-	data[6] = (data64 >> 8) & 255;
-	data[7] = (data64) & 255;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[7]; data[7] = tmp;
-		tmp = data[1]; data[1] = data[6]; data[6] = tmp;
-		tmp = data[2]; data[2] = data[5]; data[5] = tmp;
-		tmp = data[3]; data[3] = data[4]; data[4] = tmp;
-	}
-}
-
-
-/*
- *  store_32bit_word_in_host():
- *
- *  See comment for store_64bit_word_in_host().
- *
- *  (Note:  The data32 parameter is a uint64_t. This is done to suppress
- *  some warnings.)
- */
-void store_32bit_word_in_host(struct cpu *cpu,
-	unsigned char *data, uint64_t data32)
-{
-	data[0] = (data32 >> 24) & 255;
-	data[1] = (data32 >> 16) & 255;
-	data[2] = (data32 >> 8) & 255;
-	data[3] = (data32) & 255;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[3]; data[3] = tmp;
-		tmp = data[1]; data[1] = data[2]; data[2] = tmp;
-	}
-}
-
-
-/*
- *  store_16bit_word_in_host():
- *
- *  See comment for store_64bit_word_in_host().
- */
-void store_16bit_word_in_host(struct cpu *cpu,
-	unsigned char *data, uint16_t data16)
-{
-	data[0] = (data16 >> 8) & 255;
-	data[1] = (data16) & 255;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		int tmp = data[0]; data[0] = data[1]; data[1] = tmp;
-	}
-}
-
-
 /*
  *  machine_setup():
  *
@@ -895,13 +519,6 @@ void machine_setup(struct machine *machine)
 			debug(" %s", machine->bootarg);
 		debug("\n");
 	}
-
-	if (!machine->stable)
-		fatal("!\n!  NOTE: This machine type is not implemented well"
-		    " enough yet to run\n!  any real-world code!"
-		    " (At least, it hasn't been verified to do so.)\n!\n"
-		    "!  Please read the GXemul documentation for information"
-		    " about which\n!  machine types that actually work.\n!\n");
 }
 
 
@@ -930,11 +547,6 @@ void machine_memsize_fix(struct machine *m)
 		}
 	}
 
-	/*  Special hack for hpcmips machines:  */
-	if (m->machine_type == MACHINE_HPCMIPS) {
-		m->dbe_on_nonexistant_memaccess = 0;
-	}
-
 	/*  Special SGI memory offsets:  (TODO: move this somewhere else)  */
 	if (m->machine_type == MACHINE_SGI) {
 		switch (m->machine_subtype) {
@@ -1027,18 +639,17 @@ int machine_run(struct machine *machine)
 	 *  TODO: This should be redesigned into some "mainbus" stuff instead!
 	 */
 
-	machine->ninstrs += cpu0instrs;
-
-	for (te=0; te<machine->n_tick_entries; te++) {
-		machine->ticks_till_next[te] -= cpu0instrs;
-		if (machine->ticks_till_next[te] <= 0) {
-			while (machine->ticks_till_next[te] <= 0) {
-				machine->ticks_till_next[te] +=
-				    machine->ticks_reset_value[te];
+	for (te=0; te<machine->tick_functions.n_entries; te++) {
+		machine->tick_functions.ticks_till_next[te] -= cpu0instrs;
+		if (machine->tick_functions.ticks_till_next[te] <= 0) {
+			while (machine->tick_functions.ticks_till_next[te]<=0) {
+				machine->tick_functions.ticks_till_next[te] +=
+				    machine->tick_functions.
+				    ticks_reset_value[te];
 			}
 
-			machine->tick_func[te](cpus[0],
-			    machine->tick_extra[te]);
+			machine->tick_functions.f[te](cpus[0],
+			    machine->tick_functions.extra[te]);
 		}
 	}
 
@@ -1067,12 +678,7 @@ struct machine_entry *machine_entry_new(const char *name, int arch,
 {
 	struct machine_entry *me;
 
-	me = malloc(sizeof(struct machine_entry));
-	if (me == NULL) {
-		fprintf(stderr, "machine_entry_new(): out of memory (1)\n");
-		exit(1);
-	}
-
+	CHECK_ALLOCATION(me = malloc(sizeof(struct machine_entry)));
 	memset(me, 0, sizeof(struct machine_entry));
 
 	me->name = name;
@@ -1095,11 +701,9 @@ struct machine_entry *machine_entry_new(const char *name, int arch,
 void machine_entry_add_alias(struct machine_entry *me, const char *name)
 {
 	me->n_aliases ++;
-	me->aliases = realloc(me->aliases, sizeof(char *) * me->n_aliases);
-	if (me->aliases == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+
+	CHECK_ALLOCATION(me->aliases = realloc(me->aliases,
+	    sizeof(char *) * me->n_aliases));
 
 	me->aliases[me->n_aliases - 1] = (char *) name;
 }
@@ -1121,21 +725,12 @@ void machine_entry_add_subtype(struct machine_entry *me, const char *name,
 	struct machine_entry_subtype *mes;
 
 	/*  Allocate a new subtype struct:  */
-	mes = malloc(sizeof(struct machine_entry_subtype));
-	if (mes == NULL) {
-		fprintf(stderr, "machine_entry_subtype_new(): out "
-		    "of memory (1)\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(mes = malloc(sizeof(struct machine_entry_subtype)));
 
 	/*  Add the subtype to the machine entry:  */
 	me->n_subtypes ++;
-	me->subtype = realloc(me->subtype, sizeof(struct
-	    machine_entry_subtype *) * me->n_subtypes);
-	if (me->subtype == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(me->subtype = realloc(me->subtype, sizeof(struct
+	    machine_entry_subtype *) * me->n_subtypes));
 	me->subtype[me->n_subtypes - 1] = mes;
 
 	/*  Fill the struct with subtype data:  */
@@ -1155,12 +750,8 @@ void machine_entry_add_subtype(struct machine_entry *me, const char *name,
 			break;
 
 		mes->n_aliases ++;
-		mes->aliases = realloc(mes->aliases, sizeof(char *) *
-		    mes->n_aliases);
-		if (mes->aliases == NULL) {
-			fprintf(stderr, "out of memory\n");
-			exit(1);
-		}
+		CHECK_ALLOCATION(mes->aliases =
+		    realloc(mes->aliases, sizeof(char *) * mes->n_aliases));
 
 		mes->aliases[mes->n_aliases - 1] = s;
 	}
@@ -1263,13 +854,11 @@ void machine_list_available_types_and_cpus(void)
 	    "that actually work. Use the alias\nwhen selecting a machine type "
 	    "or subtype, not the real name.\n");
 
-#ifdef UNSTABLE_DEVEL
 	debug("\n");
 
 	useremul_list_emuls();
 	debug("Userland emulation works for programs with the complexity"
 	    " of Hello World,\nbut not much more.\n");
-#endif
 }
 
 
