@@ -25,7 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_hpcmips.c,v 1.6 2007/01/21 21:02:57 debug Exp $
+ *  $Id: machine_hpcmips.c,v 1.9 2007/06/15 18:08:10 debug Exp $
+ *
+ *  COMMENT: Handheld MIPS-based machines
  */
 
 #include <stdio.h>
@@ -82,7 +84,7 @@ MACHINE_SETUP(hpcmips)
 
 		/*  TODO: irq?  */
 		snprintf(tmpstr, sizeof(tmpstr), "ns16550 irq=0 addr=0x"
-		    "0a008680 addr_mult=4 in_use=%i", machine->use_x11? 0 : 1);
+		    "0a008680 addr_mult=4 in_use=%i", !machine->x11_md.in_use);
 		machine->main_console_handle = (size_t)
 		    device_add(machine, tmpstr);
 
@@ -112,7 +114,7 @@ MACHINE_SETUP(hpcmips)
 
 		/*  TODO: irq?  */
 		snprintf(tmpstr, sizeof(tmpstr), "ns16550 irq=0 addr=0x"
-		    "0a008680 addr_mult=4 in_use=%i", machine->use_x11? 0 : 1);
+		    "0a008680 addr_mult=4 in_use=%i", !machine->x11_md.in_use);
 		machine->main_console_handle = (size_t)
 		    device_add(machine, tmpstr);
 
@@ -131,7 +133,6 @@ MACHINE_SETUP(hpcmips)
 	case MACHINE_HPCMIPS_NEC_MOBILEPRO_770:
 		/*  131 MHz VR4121  */
 		machine->machine_name = "NEC MobilePro 770";
-		machine->stable = 1;
 		hpc_fb_addr = 0xa000000;
 		hpc_fb_xsize = 640;
 		hpc_fb_ysize = 240;
@@ -155,7 +156,6 @@ MACHINE_SETUP(hpcmips)
 	case MACHINE_HPCMIPS_NEC_MOBILEPRO_780:
 		/*  166 (or 168) MHz VR4121  */
 		machine->machine_name = "NEC MobilePro 780";
-		machine->stable = 1;
 		hpc_fb_addr = 0xa180100;
 		hpc_fb_xsize = 640;
 		hpc_fb_ysize = 240;
@@ -179,7 +179,6 @@ MACHINE_SETUP(hpcmips)
 	case MACHINE_HPCMIPS_NEC_MOBILEPRO_800:
 		/*  131 MHz VR4121  */
 		machine->machine_name = "NEC MobilePro 800";
-		machine->stable = 1;
 		hpc_fb_addr = 0xa000000;
 		hpc_fb_xsize = 800;
 		hpc_fb_ysize = 600;
@@ -203,7 +202,6 @@ MACHINE_SETUP(hpcmips)
 	case MACHINE_HPCMIPS_NEC_MOBILEPRO_880:
 		/*  168 MHz VR4121  */
 		machine->machine_name = "NEC MobilePro 880";
-		machine->stable = 1;
 		hpc_fb_addr = 0xa0ea600;
 		hpc_fb_xsize = 800;
 		hpc_fb_ysize = 600;
@@ -246,7 +244,7 @@ MACHINE_SETUP(hpcmips)
 			    "ns16550 irq=%i addr=0x0c000010", 8+VRIP_INTR_SIU);
 			x = (size_t)device_add(machine, tmpstr);
 
-			if (!machine->use_x11)
+			if (!machine->x11_md.in_use)
 				machine->main_console_handle = x;
 		}
 
@@ -349,7 +347,7 @@ MACHINE_SETUP(hpcmips)
 		    "res:240,bpp:4,gray,hpck:3084,inv ether=0,0x03fe0300,eth0");
 		tmp[tmplen-1] = '\0';
 
-		if (!machine->use_x11)
+		if (!machine->x11_md.in_use)
 			snprintf(tmp+strlen(tmp), tmplen-strlen(tmp),
 			    " console=ttyS0,115200");
 		tmp[tmplen-1] = '\0';
