@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_landisk.c,v 1.15 2007/04/22 03:57:38 debug Exp $
+ *  $Id: machine_landisk.c,v 1.18 2007/06/15 18:08:10 debug Exp $
  *
- *  I-O DATA LANDISK USL-5P.
+ *  COMMENT: I-O DATA LanDisk USL-5P
  *
  *  This machine consists of:
  *
@@ -41,6 +41,9 @@
  *
  *  TODO:
  *	Realtek NIC (irq 5), PCIIDE (irq 6), USB controllers, etc.
+ *
+ *  TODO 2:
+ *	Make it possible to select different landisk models.
  */
 
 #include <stdio.h>
@@ -71,7 +74,6 @@ MACHINE_SETUP(landisk)
 	char tmpstr[300];
 
 	machine->machine_name = "Landisk USL-5P";
-	machine->stable = 1;
 
 	/*  266.67 MHz SH4 CPU clock:  */
 	if (machine->emulated_hz == 0)
