@@ -25,9 +25,11 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_mvmeppc.c,v 1.18 2007/01/28 14:15:30 debug Exp $
+ *  $Id: machine_mvmeppc.c,v 1.22 2007/06/15 18:08:10 debug Exp $
  *
- *  MVMEPPC machines (for experimenting with NetBSD/mvmeppc or RTEMS).
+ *  COMMENT: MVMEPPC machines
+ *
+ *  This is for experiments with NetBSD/mvmeppc or RTEMS.
  *  (ftp://ftp.netbsd.org/pub/NetBSD/arch/mvmeppc/snapshot/20020302/README)
  *
  *  Note:  MVME machines that really adhere to the PReP standard should be
@@ -62,12 +64,6 @@ MACHINE_SETUP(mvmeppc)
 	case MACHINE_MVMEPPC_1600:
 		machine->machine_name = "MVME1600";
 
-fatal("TODO: Legacy rewrite\n");
-abort();
-//		machine->md_interrupt = isa32_interrupt;
-//		machine->md_int.prep_data = device_add(machine, "prep");
-//		machine->isa_pic_data.native_irq = 1;   /*  Semi-bogus  */
-
 		snprintf(tmpstr, sizeof(tmpstr), "eagle irq=%s.cpu[%i]",
 		    machine->path, machine->bootstrap_cpu);
 		device_add(machine, tmpstr);
@@ -186,7 +182,7 @@ MACHINE_DEFAULT_RAM(mvmeppc)
 
 MACHINE_REGISTER(mvmeppc)
 {
-	MR_DEFAULT(mvmeppc, "MVME", ARCH_PPC, MACHINE_MVMEPPC);
+	MR_DEFAULT(mvmeppc, "MVMEPPC", ARCH_PPC, MACHINE_MVMEPPC);
 
 	machine_entry_add_alias(me, "mvmeppc");
 
