@@ -25,14 +25,16 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_netwinder.c,v 1.13 2007/01/21 21:02:57 debug Exp $
+ *  $Id: machine_netwinder.c,v 1.17 2007/06/15 18:08:10 debug Exp $
+ *
+ *  COMMENT: NetWinder
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "bus_isa.h"
+#include "bus_pci.h"
 #include "cpu.h"
 #include "device.h"
 #include "devices.h"
@@ -47,7 +49,6 @@ MACHINE_SETUP(netwinder)
 	struct pci_data *pci_bus;
 
 	machine->machine_name = "NetWinder";
-	machine->stable = 1;
 
 	if (machine->physical_ram_in_mb > 256)
 		fprintf(stderr, "WARNING! Real NetWinders cannot"
@@ -60,7 +61,7 @@ MACHINE_SETUP(netwinder)
 	    " addr=0x42000000", machine->path, machine->bootstrap_cpu);
 	pci_bus = device_add(machine, tmpstr);
 
-	if (machine->use_x11) {
+	if (machine->x11_md.in_use) {
 		bus_pci_add(machine, pci_bus, machine->memory,
 		    0xc0, 8, 0, "igsfb");
 	}
