@@ -25,9 +25,9 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_pmax.c,v 1.23 2007/02/18 09:19:48 debug Exp $
+ *  $Id: machine_pmax.c,v 1.29 2007/06/15 19:11:15 debug Exp $
  *
- *  DECstation ("PMAX") machine description.
+ *  COMMENT: Digital DECstation ("PMAX") machines
  */
 
 #include <stdio.h>
@@ -39,6 +39,7 @@
 #include "devices.h"
 #include "diskimage.h"
 #include "machine.h"
+#include "machine_pmax.h"
 #include "memory.h"
 #include "misc.h"
 
@@ -136,7 +137,7 @@ MACHINE_SETUP(pmax)
 		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].%i",
 		    machine->path, machine->bootstrap_cpu, KN01_INT_DZ);
 		dev_dc7085_init(machine, mem, KN01_SYS_DZ, tmpstr,
-		    machine->use_x11);
+		    machine->x11_md.in_use);
 
 		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].%i",
 		    machine->path, machine->bootstrap_cpu, KN01_INT_CLOCK);
@@ -153,7 +154,6 @@ MACHINE_SETUP(pmax)
 		/*  Supposed to have 25MHz R3000 CPU, R3010 FPC,  */
 		/*  and a R3220 Memory coprocessor  */
 		machine->machine_name = "DECstation 5000/200 (3MAX, KN02)";
-		machine->stable = 1;
 
 		if (machine->emulated_hz == 0)
 			machine->emulated_hz = 25000000;
@@ -236,7 +236,7 @@ MACHINE_SETUP(pmax)
 		    machine->path, machine->bootstrap_cpu, 7);
 		machine->main_console_handle =
 		    dev_dc7085_init(machine, mem,
-		    KN02_SYS_DZ, tmpstr, machine->use_x11);
+		    KN02_SYS_DZ, tmpstr, machine->x11_md.in_use);
 
 		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].%i",
 		    machine->path, machine->bootstrap_cpu, KN02_INT_CLOCK);
@@ -290,9 +290,9 @@ abort();
 //		dev_le_init(machine, mem, 0x1c0c0000, 0, 0,
 //		    KMIN_INTR_LANCE + 8, 4 * 65536);
 		dev_scc_init(machine, mem, 0x1c100000, KMIN_INTR_SCC_0 + 8,
-		    machine->use_x11, 0, 1);
+		    machine->x11_md.in_use, 0, 1);
 		dev_scc_init(machine, mem, 0x1c180000, KMIN_INTR_SCC_1 + 8,
-		    machine->use_x11, 1, 1);
+		    machine->x11_md.in_use, 1, 1);
 fatal("TODO: mc146818 irq\n");
 abort();
 //		dev_mc146818_init(machine, mem, 0x1c200000, 
@@ -380,12 +380,12 @@ abort();
 //		    dev_scc_dma_func;
 //		machine->md_int.dec_ioasic_data->dma_func_extra[2] =
 //		    dev_scc_init(machine, mem, KN03_SYS_SCC_0,
-//		    KN03_INTR_SCC_0 +8, machine->use_x11, 0, 1);
+//		    KN03_INTR_SCC_0 +8, machine->x11_md.in_use, 0, 1);
 //		machine->md_int.dec_ioasic_data->dma_func[2] =
 //		    dev_scc_dma_func;
 //		machine->md_int.dec_ioasic_data->dma_func_extra[3] =
 //		    dev_scc_init(machine, mem, KN03_SYS_SCC_1,
-//		    KN03_INTR_SCC_1 +8, machine->use_x11, 1, 1);
+//		    KN03_INTR_SCC_1 +8, machine->x11_md.in_use, 1, 1);
 
 fatal("TODO: mc146818 irq\n");
 abort();
@@ -442,7 +442,7 @@ abort();
 
 		/*
 		 *  According to
-		 *  http://www2.no.netbsd.org/Ports/pmax/models.html,
+		 *  http://www2.no.netbsd.org/ports/pmax/models.html,
 		 *  the 5800-series is based on VAX 6000/300.
 		 */
 
@@ -462,7 +462,7 @@ abort();
 		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].dec5800.28",
 		    machine->path, machine->bootstrap_cpu);
 		dev_ssc_init(machine, mem, 0x10140000,
-		    tmpstr, machine->use_x11);
+		    tmpstr, machine->x11_md.in_use);
 
 		dev_decxmi_init(mem, 0x11800000);
 		dev_deccca_init(mem, DEC_DECCCA_BASEADDR);
@@ -498,9 +498,9 @@ abort();
 		 */
 		/*  ln (ethernet) at 0x10084x00 ? and 0x10120000 ?  */
 		/*  error registers (?) at 0x17000000 and 0x10080000  */
-		device_add(machine, "kn210 addr=0x10080000");
+		/*  device_add(machine, "kn210 addr=0x10080000");  */
 		dev_ssc_init(machine, mem, 0x10140000, "irq? TODO",
-		    machine->use_x11);
+		    machine->x11_md.in_use);
 		break;
 
 	case MACHINE_DEC_MAXINE_5000:	/*  type 7, KN02CA  */
@@ -581,7 +581,7 @@ abort();
 //		dev_le_init(machine, mem, 0x1c0c0000, 0, 0,
 //		    XINE_INTR_LANCE +8, 4*65536);
 		dev_scc_init(machine, mem, 0x1c100000,
-		    XINE_INTR_SCC_0 +8, machine->use_x11, 0, 1);
+		    XINE_INTR_SCC_0 +8, machine->x11_md.in_use, 0, 1);
 fatal("TODO: mc146818 irq\n");
 abort();
 //		dev_mc146818_init(machine, mem, 0x1c200000,
@@ -622,7 +622,7 @@ abort();
 		 */
 
 		dev_ssc_init(machine, mem, 0x10140000, "TODO: irq",
-		    machine->use_x11);
+		    machine->x11_md.in_use);
 
 		/*  something at 0x17000000, ultrix says "cpu 0 panic: "
 		    "DS5500 I/O Board is missing" if this is not here  */
@@ -654,7 +654,7 @@ abort();
 			    "cannot have more than 128MB RAM. Continuing"
 			    " anyway.\n");
 
-		if (machine->use_x11)
+		if (machine->x11_md.in_use)
 			fprintf(stderr, "WARNING! Real MIPSMATE 5100 machines "
 			    "cannot have a graphical framebuffer. "
 			    "Continuing anyway.\n");
@@ -679,12 +679,12 @@ abort();
 		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].kn230.0x%x",
 		    machine->path, machine->bootstrap_cpu, KN230_CSR_INTR_DZ0);
 		dev_dc7085_init(machine, mem, KN230_SYS_DZ0,
-		    tmpstr, machine->use_x11);
+		    tmpstr, machine->x11_md.in_use);
 
 		/* dev_dc7085_init(machine, mem, KN230_SYS_DZ1,
-		    KN230_CSR_INTR_OPT0, machine->use_x11);  */
+		    KN230_CSR_INTR_OPT0, machine->x11_md.in_use);  */
 		/* dev_dc7085_init(machine, mem, KN230_SYS_DZ2,
-		    KN230_CSR_INTR_OPT1, machine->use_x11);  */
+		    KN230_CSR_INTR_OPT1, machine->x11_md.in_use);  */
 
 		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].kn230.0x%x",
 		    machine->path, machine->bootstrap_cpu,
@@ -808,11 +808,7 @@ abort();
 		init_bootpath = bootpath;
 	}
 
-	machine->bootarg = malloc(BOOTARG_BUFLEN);
-	if (machine->bootarg == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(machine->bootarg = malloc(BOOTARG_BUFLEN));
 	strlcpy(machine->bootarg, init_bootpath, BOOTARG_BUFLEN);
 	if (strlcat(machine->bootarg, machine->boot_kernel_filename,
 	    BOOTARG_BUFLEN) > BOOTARG_BUFLEN) {
@@ -857,27 +853,28 @@ abort();
 
 	store_buf(cpu, BOOTINFO_ADDR, (char *)&xx, sizeof(xx));
 
+	CHECK_ALLOCATION(machine->md.pmax =
+	    malloc(sizeof(struct machine_pmax)));
+	memset(machine->md.pmax, 0, sizeof(struct machine_pmax));
+
 	/*  The system's memmap:  */
-	machine->md.pmax.memmap = malloc(sizeof(struct dec_memmap));
-	if (machine->md.pmax.memmap == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(1);
-	}
+	CHECK_ALLOCATION(machine->md.pmax->memmap =
+	    malloc(sizeof(struct dec_memmap)));
 	store_32bit_word_in_host(cpu,
-	    (unsigned char *)&machine->md.pmax.memmap->pagesize, 4096);
+	    (unsigned char *)&machine->md.pmax->memmap->pagesize, 4096);
 	{
 		unsigned int i;
-		for (i=0; i<sizeof(machine->md.pmax.memmap->bitmap); i++)
-			machine->md.pmax.memmap->bitmap[i] = ((int)i * 4096*8 <
+		for (i=0; i<sizeof(machine->md.pmax->memmap->bitmap); i++)
+			machine->md.pmax->memmap->bitmap[i] = ((int)i * 4096*8 <
 			    1048576*machine->physical_ram_in_mb)? 0xff : 0x00;
 	}
 	store_buf(cpu, DEC_MEMMAP_ADDR,
-	    (char *)machine->md.pmax.memmap, sizeof(struct dec_memmap));
+	    (char *)machine->md.pmax->memmap, sizeof(struct dec_memmap));
 
 	/*  Environment variables:  */
 	addr = DEC_PROM_STRINGS;
 
-	if (machine->use_x11 && machine->n_gfx_cards > 0)
+	if (machine->x11_md.in_use && machine->n_gfx_cards > 0)
 		/*  (0,3)  Keyboard and Framebuffer  */
 		add_environment_string(cpu, framebuffer_console_name, &addr);
 	else
@@ -938,13 +935,13 @@ abort();
 MACHINE_DEFAULT_CPU(pmax)
 {
 	if (machine->machine_subtype > 2)
-		machine->cpu_name = strdup("R3000A");
+		CHECK_ALLOCATION(machine->cpu_name = strdup("R3000A"));
 
 	if (machine->machine_subtype > 1 && machine->cpu_name == NULL)
-		machine->cpu_name = strdup("R3000");
+		CHECK_ALLOCATION(machine->cpu_name = strdup("R3000"));
 
 	if (machine->cpu_name == NULL)
-		machine->cpu_name = strdup("R2000");
+		CHECK_ALLOCATION(machine->cpu_name = strdup("R2000"));
 }
 
 
