@@ -25,10 +25,27 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_test.c,v 1.33 2007/04/19 15:18:16 debug Exp $
+ *  $Id: machine_test.c,v 1.38 2007/06/15 18:08:10 debug Exp $
  *
- *  Various "test" machines (bare machines with just a CPU, or a bare machine
- *  plus some experimental devices).
+ *  COMMENT: Various test machines
+ *
+ *  Generally, the machines are as follows:
+ *
+ *	bareXYZ:	A bare machine using an XYZ processor.
+ *
+ *	testXYZ:	A machine with an XYZ processor, and some experimental
+ *			devices connected to it.
+ *
+ *  The experimental devices in the test machines are:
+ *
+ *	cons		A serial I/O console device.
+ *	disk		A device for reading/writing (emulated) disk sectors.
+ *	ether		An ethernet device, for sending/receiving ethernet
+ *			frames on an emulated network.
+ *	fb		Framebuffer (24-bit RGB per pixel).
+ *	irqc		A generic interrupt controller.
+ *	mp		A multiprocessor controller.
+ *	rtc		A real-time clock device.
  */
 
 #include <stdio.h>
@@ -40,24 +57,72 @@
 #include "memory.h"
 #include "misc.h"
 
+#include "sh4_exception.h"
+
 #include "testmachine/dev_cons.h"
 #include "testmachine/dev_disk.h"
 #include "testmachine/dev_ether.h"
 #include "testmachine/dev_fb.h"
+#include "testmachine/dev_irqc.h"
 #include "testmachine/dev_mp.h"
 #include "testmachine/dev_rtc.h"
 
 
+/*
+ *  default_test():
+ *
+ *  Initializes devices for most test machines. (Note: MIPS is different,
+ *  because of legacy reasons.)
+ */
 static void default_test(struct machine *machine, struct cpu *cpu)
 {
 	char tmpstr[1000];
+	char base_irq[1000];
+	char end_of_base_irq[50];
 
-	snprintf(tmpstr, sizeof(tmpstr), "cons addr=0x%"PRIx64" irq=none",
-	    (uint64_t) DEV_CONS_ADDRESS);
+	/*
+	 *  First add the interrupt controller. Most processor architectures
+	 *  in GXemul have only 1 interrupt pin on the CPU, and it is simply
+	 *  called "emul[x].machine[y].cpu[z]".
+	 *
+	 *  MIPS is an exception, dealt with in a separate setup function.
+	 *  ARM and SH are dealt with here.
+	 */
+
+	switch (machine->arch) {
+
+	case ARCH_ARM:
+		snprintf(end_of_base_irq, sizeof(end_of_base_irq), ".irq");
+		break;
+
+	case ARCH_SH:
+		snprintf(end_of_base_irq, sizeof(end_of_base_irq),
+		    ".irq[0x%x]", SH4_INTEVT_IRQ15);
+		break;
+
+	default:
+		end_of_base_irq[0] = '\0';
+	}
+
+	snprintf(base_irq, sizeof(base_irq), "%s.cpu[%i]%s",
+	    machine->path, machine->bootstrap_cpu, end_of_base_irq);
+
+	snprintf(tmpstr, sizeof(tmpstr), "irqc addr=0x%"PRIx64" irq=%s",
+	    (uint64_t) DEV_IRQC_ADDRESS, base_irq);
+	device_add(machine, tmpstr);
+
+
+	/*  Now, add the other devices:  */
+
+	snprintf(tmpstr, sizeof(tmpstr), "cons addr=0x%"PRIx64
+	    " irq=%s.irqc.2 in_use=%i",
+	    (uint64_t) DEV_CONS_ADDRESS, base_irq, machine->arch != ARCH_SH);
 	machine->main_console_handle = (size_t)device_add(machine, tmpstr);
 
-	snprintf(tmpstr, sizeof(tmpstr), "mp addr=0x%"PRIx64,
-	    (uint64_t) DEV_MP_ADDRESS);
+	snprintf(tmpstr, sizeof(tmpstr), "mp addr=0x%"PRIx64" irq=%s%sirqc.6",
+	    (uint64_t) DEV_MP_ADDRESS,
+	    end_of_base_irq[0]? end_of_base_irq + 1 : "",
+	    end_of_base_irq[0]? "." : "");
 	device_add(machine, tmpstr);
 
 	snprintf(tmpstr, sizeof(tmpstr), "fbctrl addr=0x%"PRIx64,
@@ -68,12 +133,12 @@ static void default_test(struct machine *machine, struct cpu *cpu)
 	    (uint64_t) DEV_DISK_ADDRESS);
 	device_add(machine, tmpstr);
 
-	snprintf(tmpstr, sizeof(tmpstr), "ether addr=0x%"PRIx64" irq=none",
-	    (uint64_t) DEV_ETHER_ADDRESS);
+	snprintf(tmpstr, sizeof(tmpstr), "ether addr=0x%"PRIx64" irq=%s.irqc.3",
+	    (uint64_t) DEV_ETHER_ADDRESS, base_irq);
 	device_add(machine, tmpstr);
 
-	snprintf(tmpstr, sizeof(tmpstr), "rtc addr=0x%"PRIx64" irq=none",
-	    (uint64_t) DEV_RTC_ADDRESS);
+	snprintf(tmpstr, sizeof(tmpstr), "rtc addr=0x%"PRIx64" irq=%s.irqc.4",
+	    (uint64_t) DEV_RTC_ADDRESS, base_irq);
 	device_add(machine, tmpstr);
 }
 
@@ -81,16 +146,12 @@ static void default_test(struct machine *machine, struct cpu *cpu)
 MACHINE_SETUP(barealpha)
 {
 	machine->machine_name = "Generic \"bare\" Alpha machine";
-	machine->stable = 1;
 }
 
 
 MACHINE_SETUP(testalpha)
 {
 	machine->machine_name = "Alpha test machine";
-	machine->stable = 1;
-
-	/*  TODO: interrupt for Alpha?  */
 
 	default_test(machine, cpu);
 }
@@ -129,16 +190,12 @@ MACHINE_REGISTER(testalpha)
 MACHINE_SETUP(barearm)
 {
 	machine->machine_name = "Generic \"bare\" ARM machine";
-	machine->stable = 1;
 }
 
 
 MACHINE_SETUP(testarm)
 {
 	machine->machine_name = "ARM test machine";
-	machine->stable = 1;
-
-	/*  TODO: interrupt for ARM?  */
 
 	default_test(machine, cpu);
 
@@ -188,16 +245,12 @@ MACHINE_REGISTER(testarm)
 MACHINE_SETUP(barem88k)
 {
 	machine->machine_name = "Generic \"bare\" M88K machine";
-	machine->stable = 1;
 }
 
 
 MACHINE_SETUP(testm88k)
 {
 	machine->machine_name = "M88K test machine";
-	machine->stable = 1;
-
-	/*  TODO: interrupt for M88K?  */
 
 	default_test(machine, cpu);
 }
@@ -236,7 +289,6 @@ MACHINE_REGISTER(testm88k)
 MACHINE_SETUP(baremips)
 {
 	machine->machine_name = "Generic \"bare\" MIPS machine";
-	machine->stable = 1;
 	cpu->byte_order = EMUL_BIG_ENDIAN;
 }
 
@@ -261,7 +313,6 @@ MACHINE_SETUP(testmips)
 	char tmpstr[300];
 
 	machine->machine_name = "MIPS test machine";
-	machine->stable = 1;
 	cpu->byte_order = EMUL_BIG_ENDIAN;
 
 	snprintf(tmpstr, sizeof(tmpstr), "cons addr=0x%"PRIx64" irq=%s."
@@ -269,7 +320,7 @@ MACHINE_SETUP(testmips)
 	    machine->bootstrap_cpu);
 	machine->main_console_handle = (size_t)device_add(machine, tmpstr);
 
-	snprintf(tmpstr, sizeof(tmpstr), "mp irq=6 addr=0x%"PRIx64,
+	snprintf(tmpstr, sizeof(tmpstr), "mp addr=0x%"PRIx64" irq=6",
 	    (uint64_t) DEV_MP_ADDRESS);
 	device_add(machine, tmpstr);
 
@@ -326,16 +377,12 @@ MACHINE_REGISTER(testmips)
 MACHINE_SETUP(bareppc)
 {
 	machine->machine_name = "Generic \"bare\" PPC machine";
-	machine->stable = 1;
 }
 
 
 MACHINE_SETUP(testppc)
 {
 	machine->machine_name = "PPC test machine";
-	machine->stable = 1;
-
-	/*  TODO: interrupt for PPC?  */
 
 	default_test(machine, cpu);
 }
@@ -373,16 +420,12 @@ MACHINE_REGISTER(testppc)
 MACHINE_SETUP(baresh)
 {
 	machine->machine_name = "Generic \"bare\" SH machine";
-	machine->stable = 1;
 }
 
 
 MACHINE_SETUP(testsh)
 {
 	machine->machine_name = "SH test machine";
-	machine->stable = 1;
-
-	/*  TODO: interrupt for SH?  */
 
 	default_test(machine, cpu);
 }
@@ -420,16 +463,12 @@ MACHINE_REGISTER(testsh)
 MACHINE_SETUP(baresparc)
 {
 	machine->machine_name = "Generic \"bare\" SPARC machine";
-	machine->stable = 1;
 }
 
 
 MACHINE_SETUP(testsparc)
 {
 	machine->machine_name = "SPARC test machine";
-	machine->stable = 1;
-
-	/*  TODO: interrupt for SPARC?  */
 
 	default_test(machine, cpu);
 }
