@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: misc.c,v 1.6 2006/12/30 13:30:52 debug Exp $
+ *  $Id: misc.c,v 1.9 2007/06/15 17:02:38 debug Exp $
  *
  *  This file contains things that don't fit anywhere else, and fake/dummy
  *  implementations of libc functions that are missing on some systems.
@@ -36,6 +36,7 @@
 #include <string.h>
 #include <fcntl.h>
 
+#include "cpu.h"
 #include "misc.h"
 
 
@@ -156,3 +157,17 @@ size_t mystrlcat(char *dst, const char *src, size_t size)
 }
 #endif
 
+
+/*
+ *  print_separator_line():
+ *
+ *  Prints a line of "----------".
+ */
+void print_separator_line(void)
+{
+        int i = 79; 
+        while (i-- > 0)
+                debug("-");
+        debug("\n");
+}
+
